@@ -23,7 +23,9 @@ module Language.Haskell.Exts.Annotated.Simplify where
 import Language.Haskell.Exts.Annotated.Syntax
 import qualified Language.Haskell.Exts.Syntax as S
 
-import Language.Haskell.Exts.SrcLoc
+import Language.Haskell.Exts.SrcLoc hiding (loc)
+
+import Data.Maybe (fromMaybe)
 
 -- | Translate an annotated AST node representing a Haskell module, into
 --   a simpler version that retains (almost) only abstract information.
@@ -38,7 +40,7 @@ sModule md = case md of
         let loc = getPointLoc l
          in S.Module loc (sModuleName mn) (map sModulePragma oss)
                       Nothing
-                      (Just [S.EVar $ S.UnQual $ S.Ident "page"])
+                      (Just [S.EVar S.NoNamespace $ S.UnQual $ S.Ident "page"])
                         []
                         [pageFun loc $ S.XTag loc (sXName xn) (map sXAttr attrs) (fmap sExp mat) (map sExp es)]
     XmlHybrid l mmh oss ids ds xn attrs mat es  ->
@@ -48,7 +50,7 @@ sModule md = case md of
          in S.Module loc1 mn (map sModulePragma oss) mwt mes (map sImportDecl ids)
                 (map sDecl ds ++ [pageFun loc2 $ S.XTag loc2 (sXName xn) (map sXAttr attrs) (fmap sExp mat) (map sExp es)])
   where pageFun :: SrcLoc -> S.Exp -> S.Decl
-        pageFun loc e = S.PatBind loc namePat Nothing rhs (S.BDecls [])
+        pageFun loc e = S.PatBind loc namePat rhs (S.BDecls [])
             where namePat = S.PVar $ S.Ident "page"
                   rhs = S.UnGuardedRhs e
 
@@ -64,6 +66,9 @@ sDecl decl = case decl of
      TypeFamDecl  l dh mk       ->
         let (n, tvs) = sDeclHead dh
          in S.TypeFamDecl (getPointLoc l) n tvs (fmap sKind mk)
+     ClosedTypeFamDecl  l dh mk eqs ->
+        let (n, tvs) = sDeclHead dh
+         in S.ClosedTypeFamDecl (getPointLoc l) n tvs (fmap sKind mk) (map sTypeEqn eqs)
      DataDecl     l dn mctxt dh constrs mder    ->
         let (n, tvs) = sDeclHead dh
          in S.DataDecl (getPointLoc l) (sDataOrNew dn) (maybe [] sContext mctxt) n tvs (map sQualConDecl constrs) (maybe [] sDeriving mder)
@@ -81,23 +86,23 @@ sDecl decl = case decl of
      ClassDecl    l mctxt dh fds mcds           ->
         let (n, tvs) = sDeclHead dh
          in S.ClassDecl (getPointLoc l) (maybe [] sContext mctxt) n tvs (map sFunDep fds) (maybe [] (map sClassDecl) mcds)
-     InstDecl     l mctxt ih mids               ->
-        let (qn, ts) = sInstHead ih
-         in S.InstDecl (getPointLoc l) (maybe [] sContext mctxt) qn ts (maybe [] (map sInstDecl) mids)
-     DerivDecl    l mctxt ih    ->
-        let (qn, ts) = sInstHead ih
-         in S.DerivDecl (getPointLoc l) (maybe [] sContext mctxt) qn ts
-     InfixDecl    l ass prec ops    -> S.InfixDecl (getPointLoc l) (sAssoc ass) (maybe 9 id prec) (map sOp ops)
+     InstDecl     l olp ih mids               ->
+        let (tvs, cxt, (qn, ts)) = sInstRule ih
+         in S.InstDecl (getPointLoc l) (fmap sOverlap olp) tvs cxt qn ts (maybe [] (map sInstDecl) mids)
+     DerivDecl    l olp ih    ->
+        let (tvs, cxt, (qn, ts)) = sInstRule ih
+         in S.DerivDecl (getPointLoc l) (fmap sOverlap olp) tvs cxt qn ts
+     InfixDecl    l ass prec ops    -> S.InfixDecl (getPointLoc l) (sAssoc ass) (fromMaybe 9 prec) (map sOp ops)
      DefaultDecl  l ts          -> S.DefaultDecl (getPointLoc l) (map sType ts)
      SpliceDecl   l sp          -> S.SpliceDecl (getPointLoc l) (sExp sp)
      TypeSig      l ns t        -> S.TypeSig (getPointLoc l) (map sName ns) (sType t)
      FunBind      _ ms          -> S.FunBind (map sMatch ms)
-     PatBind      l p mt rhs mbs    ->
-        S.PatBind (getPointLoc l) (sPat p) (fmap sType mt) (sRhs rhs) (maybe (S.BDecls []) sBinds mbs)
+     PatBind      l p rhs mbs    ->
+        S.PatBind (getPointLoc l) (sPat p) (sRhs rhs) (maybe (S.BDecls []) sBinds mbs)
      ForImp       l cc msaf mstr n t    ->
-        S.ForImp (getPointLoc l) (sCallConv cc) (maybe (S.PlaySafe False) sSafety msaf) (maybe "" id mstr) (sName n) (sType t)
+        S.ForImp (getPointLoc l) (sCallConv cc) (maybe (S.PlaySafe False) sSafety msaf) (fromMaybe "" mstr) (sName n) (sType t)
      ForExp       l cc      mstr n t    ->
-        S.ForExp (getPointLoc l) (sCallConv cc) (maybe "" id mstr) (sName n) (sType t)
+        S.ForExp (getPointLoc l) (sCallConv cc) (fromMaybe "" mstr) (sName n) (sType t)
      RulePragmaDecl   l rs      -> S.RulePragmaDecl (getPointLoc l) (map sRule rs)
      DeprPragmaDecl   l nsstrs  -> S.DeprPragmaDecl (getPointLoc l) (map (\(ns, str) -> (map sName ns, str)) nsstrs)
      WarnPragmaDecl   l nsstrs  -> S.WarnPragmaDecl (getPointLoc l) (map (\(ns, str) -> (map sName ns, str)) nsstrs)
@@ -106,18 +111,30 @@ sDecl decl = case decl of
      SpecSig          l   mact qn ts   -> S.SpecSig (getPointLoc l) (maybe S.AlwaysActive sActivation mact) (sQName qn) (map sType ts)
      SpecInlineSig    l b mact qn ts    ->
         S.SpecInlineSig (getPointLoc l) b (maybe S.AlwaysActive sActivation mact) (sQName qn) (map sType ts)
-     InstSig          l mctxt ih    ->
-        let (qn, ts) = sInstHead ih
-         in S.InstSig (getPointLoc l) (maybe [] sContext mctxt) qn ts
-     AnnPragma        l ann         ->
-        S.AnnPragma (getPointLoc l) (sAnnotation ann)
+     InstSig          l ih    ->
+        let (tvs, cxt, (qn, ts)) = sInstRule ih
+         in S.InstSig (getPointLoc l) tvs cxt qn ts
+     AnnPragma        l ann'        ->
+        S.AnnPragma (getPointLoc l) (sAnnotation ann')
+     MinimalPragma    l b           ->
+        S.MinimalPragma (getPointLoc l) (fmap sBooleanFormula b)
+
+sTypeEqn :: SrcInfo l => TypeEqn l -> S.TypeEqn
+sTypeEqn (TypeEqn _ a b) = S.TypeEqn (sType a) (sType b)
 
 sAnnotation :: SrcInfo loc => Annotation loc -> S.Annotation
-sAnnotation ann = case ann of
+sAnnotation ann' = case ann' of
     Ann       _ n e   -> S.Ann     (sName n) (sExp e)
     TypeAnn   _ n e   -> S.TypeAnn (sName n) (sExp e)
     ModuleAnn _   e   -> S.ModuleAnn         (sExp e)
 
+sBooleanFormula :: SrcInfo loc => BooleanFormula loc -> S.BooleanFormula
+sBooleanFormula b' = case b' of
+    VarFormula _ n   -> S.VarFormula (sName n)
+    AndFormula _ ns  -> S.AndFormula $ map sBooleanFormula ns
+    OrFormula _  ns  -> S.OrFormula $ map sBooleanFormula ns
+    ParenFormula _ b -> S.ParenFormula (sBooleanFormula b)
+
 sModuleName :: ModuleName l -> S.ModuleName
 sModuleName (ModuleName _ str)  = S.ModuleName str
 
@@ -156,9 +173,9 @@ sCName :: CName l -> S.CName
 sCName (VarName _ n) = S.VarName (sName n)
 sCName (ConName _ n) = S.ConName (sName n)
 
-sModuleHead :: Maybe (ModuleHead l) -> (S.ModuleName, Maybe (S.WarningText), Maybe [S.ExportSpec])
+sModuleHead :: Maybe (ModuleHead l) -> (S.ModuleName, Maybe S.WarningText, Maybe [S.ExportSpec])
 sModuleHead mmh = case mmh of
-    Nothing -> (S.main_mod, Nothing, Just [S.EVar (S.UnQual S.main_name)])
+    Nothing -> (S.main_mod, Nothing, Just [S.EVar S.NoNamespace (S.UnQual S.main_name)])
     Just (ModuleHead _ mn mwt mel) -> (sModuleName mn, fmap sWarningText mwt, fmap sExportSpecList mel)
 
 sExportSpecList :: ExportSpecList l -> [S.ExportSpec]
@@ -166,22 +183,27 @@ sExportSpecList (ExportSpecList _ ess) = map sExportSpec ess
 
 sExportSpec :: ExportSpec l -> S.ExportSpec
 sExportSpec es = case es of
-    EVar _ qn           -> S.EVar (sQName qn)
+    EVar _ n qn         -> S.EVar (sNamespace n) (sQName qn)
     EAbs _ qn           -> S.EAbs (sQName qn)
     EThingAll _ qn      -> S.EThingAll (sQName qn)
     EThingWith _ qn cns -> S.EThingWith (sQName qn) (map sCName cns)
     EModuleContents _ mn    -> S.EModuleContents (sModuleName mn)
 
 sImportDecl :: SrcInfo loc => ImportDecl loc -> S.ImportDecl
-sImportDecl (ImportDecl l mn qu src mpkg as misl) =
-    S.ImportDecl (getPointLoc l) (sModuleName mn) qu src mpkg (fmap sModuleName as) (fmap sImportSpecList misl)
+sImportDecl (ImportDecl l mn qu src safe mpkg as misl) =
+    S.ImportDecl (getPointLoc l) (sModuleName mn) qu src safe mpkg (fmap sModuleName as) (fmap sImportSpecList misl)
 
 sImportSpecList :: ImportSpecList l -> (Bool, [S.ImportSpec])
 sImportSpecList (ImportSpecList _ b iss) = (b, map sImportSpec iss)
 
+sNamespace :: Namespace l -> S.Namespace
+sNamespace n = case n of
+                NoNamespace _   -> S.NoNamespace
+                TypeNamespace _ -> S.TypeNamespace
+
 sImportSpec :: ImportSpec l -> S.ImportSpec
 sImportSpec is = case is of
-    IVar _ n            -> S.IVar (sName n)
+    IVar _ ns n          -> S.IVar (sNamespace ns) (sName n)
     IAbs _ n            -> S.IAbs (sName n)
     IThingAll _ n       -> S.IThingAll (sName n)
     IThingWith _ n cns  -> S.IThingWith (sName n) (map sCName cns)
@@ -193,23 +215,36 @@ sAssoc a = case a of
     AssocRight _ -> S.AssocRight
 
 sDeclHead :: DeclHead l -> (S.Name, [S.TyVarBind])
-sDeclHead dh = case dh of
-    DHead _ n tvs       -> (sName n, map sTyVarBind tvs)
-    DHInfix _ tva n tvb -> (sName n, map sTyVarBind [tva,tvb])
-    DHParen _ dh        -> sDeclHead dh
+sDeclHead = go []
+  where go ts dh' = case dh' of
+                        DHead _ n           -> (sName n, ts)
+                        DHInfix _ tva n     -> (sName n, sTyVarBind tva:ts)
+                        DHParen _ dh        -> (n, ts1 ++ ts)
+                          where (n, ts1) = sDeclHead dh
+                        DHApp _ dh t        -> go (sTyVarBind t:ts) dh
 
-sInstHead :: InstHead l -> (S.QName, [S.Type])
-sInstHead ih = case ih of
-    IHead _ qn ts      -> (sQName qn, map sType ts)
-    IHInfix _ ta qn tb -> (sQName qn, map sType [ta,tb])
-    IHParen _ ih       -> sInstHead ih
+sInstRule :: SrcInfo l => InstRule l -> ([S.TyVarBind], [S.Asst], (S.QName, [S.Type]))
+sInstRule ih' = case ih' of
+                    IRule _ mtvs mctxt qn   -> (tvs, cxt, sInstHead qn)
+                        where cxt = maybe [] sContext mctxt
+                              tvs = maybe [] (map sTyVarBind) mtvs
+                    IParen _ ir             -> sInstRule ir
+
+sInstHead :: SrcInfo l => InstHead l -> (S.QName, [S.Type])
+sInstHead = go []
+    where go ts' d' = case d' of
+                        IHCon _ qn         -> (sQName qn, ts')
+                        IHInfix _ ta qn    -> (sQName qn, sType ta:ts')
+                        IHParen _ ih       -> (n, ts1 ++ ts')
+                          where (n, ts1) = sInstHead ih
+                        IHApp _ ih t       -> go (sType t:ts') ih
 
 sDataOrNew :: DataOrNew l -> S.DataOrNew
 sDataOrNew (DataType _) = S.DataType
 sDataOrNew (NewType _) = S.NewType
 
-sDeriving :: (Deriving l) -> [(S.QName, [S.Type])]
-sDeriving (Deriving _ ihs) = map sInstHead ihs
+sDeriving :: SrcInfo l => Deriving l -> [(S.QName, [S.Type])]
+sDeriving (Deriving _ irs) = map (\ir -> let (_, _, ret) = sInstRule ir in ret) irs
 
 sBinds :: SrcInfo loc => Binds loc -> S.Binds
 sBinds bs = case bs of
@@ -229,17 +264,17 @@ sQualConDecl :: SrcInfo loc => QualConDecl loc -> S.QualConDecl
 sQualConDecl (QualConDecl l mtvs mctxt cd) =
     S.QualConDecl (getPointLoc l) (maybe [] (map sTyVarBind) mtvs) (maybe [] sContext mctxt) (sConDecl cd)
 
-sConDecl :: ConDecl l -> S.ConDecl
+sConDecl :: SrcInfo l => ConDecl l -> S.ConDecl
 sConDecl cd = case cd of
-    ConDecl _ n bts     -> S.ConDecl (sName n) (map sBangType bts)
-    InfixConDecl _ bta n btb -> S.InfixConDecl (sBangType bta) (sName n) (sBangType btb)
+    ConDecl _ n bts     -> S.ConDecl (sName n) (map sType bts)
+    InfixConDecl _ bta n btb -> S.InfixConDecl (sType bta) (sName n) (sType btb)
     RecDecl _ n fds -> S.RecDecl (sName n) (map sFieldDecl fds)
 
-sFieldDecl :: FieldDecl l -> ([S.Name], S.BangType)
-sFieldDecl (FieldDecl _ ns bt) = (map sName ns, sBangType bt)
+sFieldDecl :: SrcInfo l => FieldDecl l -> ([S.Name], S.Type)
+sFieldDecl (FieldDecl _ ns bt) = (map sName ns, sType bt)
 
 sGadtDecl :: SrcInfo loc => GadtDecl loc -> S.GadtDecl
-sGadtDecl (GadtDecl l n t) = S.GadtDecl (getPointLoc l) (sName n) (sType t)
+sGadtDecl (GadtDecl l n mn t) = S.GadtDecl (getPointLoc l) (sName n) (maybe [] sRecFields mn) (sType t)
 
 sClassDecl :: SrcInfo loc => ClassDecl loc -> S.ClassDecl
 sClassDecl cd = case cd of
@@ -252,9 +287,14 @@ sClassDecl cd = case cd of
          in S.ClsTyFam (getPointLoc l) n tvs (fmap sKind mk)
     ClsTyDef l t1 t2    ->
         S.ClsTyDef (getPointLoc l) (sType t1) (sType t2)
+    ClsDefSig l n t ->
+        S.ClsDefSig (getPointLoc l) (sName n) (sType t)
+
+sRecFields :: SrcInfo l => [FieldDecl l] -> [([S.Name], S.Type)]
+sRecFields = map sFieldDecl
 
 sInstDecl :: SrcInfo loc => InstDecl loc -> S.InstDecl
-sInstDecl id = case id of
+sInstDecl id' = case id' of
     InsDecl   _ d   -> S.InsDecl (sDecl d)
     InsType   l t1 t2   -> S.InsType (getPointLoc l) (sType t1) (sType t2)
     InsData   l dn t constrs mder   ->
@@ -263,11 +303,10 @@ sInstDecl id = case id of
         S.InsGData (getPointLoc l) (sDataOrNew dn) (sType t) (fmap sKind mk) (map sGadtDecl gds) (maybe [] sDeriving mder)
 --    InsInline l b mact qn   -> S.InsInline (getPointLoc l) b (maybe S.AlwaysActive sActivation mact) (sQName qn)
 
-sBangType :: BangType l -> S.BangType
+sBangType :: SrcInfo l => BangType l -> S.BangType
 sBangType bt = case bt of
-    BangedTy   _ t  -> S.BangedTy (sType t)
-    UnBangedTy _ t  -> S.UnBangedTy (sType t)
-    UnpackedTy _ t  -> S.UnpackedTy (sType t)
+    BangedTy   _  -> S.BangedTy
+    UnpackedTy _  -> S.UnpackedTy
 
 sRhs :: SrcInfo loc => Rhs loc -> S.Rhs
 sRhs (UnGuardedRhs _ e) = S.UnGuardedRhs (sExp e)
@@ -276,47 +315,66 @@ sRhs (GuardedRhss _ grhss) = S.GuardedRhss (map sGuardedRhs grhss)
 sGuardedRhs :: SrcInfo loc => GuardedRhs loc -> S.GuardedRhs
 sGuardedRhs (GuardedRhs l ss e) = S.GuardedRhs (getPointLoc l) (map sStmt ss) (sExp e)
 
-sType :: Type l -> S.Type
-sType t = case t of
+sType :: SrcInfo l => Type l -> S.Type
+sType t' = case t' of
     TyForall _ mtvs mctxt t     -> S.TyForall (fmap (map sTyVarBind) mtvs) (maybe [] sContext mctxt) (sType t)
     TyFun _ t1 t2               -> S.TyFun (sType t1) (sType t2)
     TyTuple _ bx ts             -> S.TyTuple bx (map sType ts)
     TyList _ t                  -> S.TyList (sType t)
+    TyParArray _ t              -> S.TyParArray (sType t)
     TyApp _ t1 t2               -> S.TyApp (sType t1) (sType t2)
     TyVar _ n                   -> S.TyVar (sName n)
     TyCon _ qn                  -> S.TyCon (sQName qn)
     TyParen _ t                 -> S.TyParen (sType t)
     TyInfix _ ta qn tb          -> S.TyInfix (sType ta) (sQName qn) (sType tb)
     TyKind _ t k                -> S.TyKind (sType t) (sKind k)
+    TyPromoted _ t              -> S.TyPromoted (sPromoted t)
+    TyEquals _ t1 t2            -> S.TyEquals (sType t1) (sType t2)
+    TySplice _ s                -> S.TySplice (sSplice s)
+    TyBang _ b t                -> S.TyBang (sBangType b) (sType t)
+
+sPromoted :: Promoted l -> S.Promoted
+sPromoted p = case p of
+    PromotedInteger _ n _ -> S.PromotedInteger n
+    PromotedString _ s _ -> S.PromotedString s
+    PromotedCon _ b qn -> S.PromotedCon b (sQName qn)
+    PromotedList _ b ps -> S.PromotedList b (map sPromoted ps)
+    PromotedTuple _ ps -> S.PromotedTuple (map sPromoted ps)
+    PromotedUnit _ -> S.PromotedUnit
+
 
 sTyVarBind :: TyVarBind l -> S.TyVarBind
 sTyVarBind (KindedVar _ n k) = S.KindedVar (sName n) (sKind k)
 sTyVarBind (UnkindedVar _ n) = S.UnkindedVar (sName n)
 
 sKind :: Kind l -> S.Kind
-sKind k = case k of
+sKind k' = case k' of
     KindStar  _     -> S.KindStar
     KindBang  _     -> S.KindBang
     KindFn _ k1 k2  -> S.KindFn (sKind k1) (sKind k2)
     KindParen _ k   -> S.KindParen (sKind k)
-    KindVar _ n     -> S.KindVar (sName n)
+    KindVar _ n     -> S.KindVar (sQName n)
+    KindApp _ k1 k2 -> S.KindApp (sKind k1) (sKind k2)
+    KindTuple _ ks  -> S.KindTuple (map sKind ks)
+    KindList  _ ks  -> S.KindList  (map sKind ks)
 
 sFunDep :: FunDep l -> S.FunDep
 sFunDep (FunDep _ as bs) = S.FunDep (map sName as) (map sName bs)
 
-sContext :: Context l -> S.Context
+sContext :: SrcInfo l => Context l -> S.Context
 sContext ctxt = case ctxt of
     CxSingle _ asst     -> [sAsst asst]
     CxTuple  _ assts    -> map sAsst assts
-    CxParen  _ ct       -> sContext ct
     CxEmpty  _          -> []
 
-sAsst :: Asst l -> S.Asst
+sAsst :: SrcInfo l => Asst l -> S.Asst
 sAsst asst = case asst of
     ClassA _ qn ts      -> S.ClassA (sQName qn) (map sType ts)
+    VarA _ n            -> S.VarA (sName n)
     InfixA _ ta qn tb   -> S.InfixA (sType ta) (sQName qn) (sType tb)
     IParam _ ipn t      -> S.IParam (sIPName ipn) (sType t)
     EqualP _ t1 t2      -> S.EqualP (sType t1) (sType t2)
+    ParenA _ a          -> S.ParenA (sAsst a)
 
 sLiteral :: Literal l -> S.Literal
 sLiteral lit = case lit of
@@ -331,8 +389,13 @@ sLiteral lit = case lit of
     PrimChar   _ c _ -> S.PrimChar c
     PrimString _ s _ -> S.PrimString s
 
+sSign :: Sign l -> S.Sign
+sSign sg = case sg of
+    Signless _ -> S.Signless
+    Negative _ -> S.Negative
+
 sExp :: SrcInfo loc => Exp loc -> S.Exp
-sExp e = case e of
+sExp e' = case e' of
     Var _ qn            -> S.Var (sQName qn)
     IPVar _ ipn         -> S.IPVar (sIPName ipn)
     Con _ qn            -> S.Con (sQName qn)
@@ -343,12 +406,14 @@ sExp e = case e of
     Lambda l ps e       -> S.Lambda (getPointLoc l) (map sPat ps) (sExp e)
     Let _ bs e          -> S.Let (sBinds bs) (sExp e)
     If _ e1 e2 e3       -> S.If (sExp e1) (sExp e2) (sExp e3)
+    MultiIf _ alts      -> S.MultiIf (map sGuardedRhs alts)
     Case _ e alts       -> S.Case (sExp e) (map sAlt alts)
     Do _ ss             -> S.Do (map sStmt ss)
     MDo _ ss            -> S.MDo (map sStmt ss)
     Tuple _ bx es       -> S.Tuple bx (map sExp es)
     TupleSection _ bx mes -> S.TupleSection bx (map (fmap sExp) mes)
     List _ es           -> S.List (map sExp es)
+    ParArray _ es       -> S.ParArray (map sExp es)
     Paren _ e           -> S.Paren (sExp e)
     LeftSection _ e op  -> S.LeftSection (sExp e) (sQOp op)
     RightSection _ op e -> S.RightSection (sQOp op) (sExp e)
@@ -358,8 +423,11 @@ sExp e = case e of
     EnumFromTo _ e1 e2  -> S.EnumFromTo (sExp e1) (sExp e2)
     EnumFromThen _ e1 e2    -> S.EnumFromThen (sExp e1) (sExp e2)
     EnumFromThenTo _ e1 e2 e3   -> S.EnumFromThenTo (sExp e1) (sExp e2) (sExp e3)
+    ParArrayFromTo _ e1 e2 -> S.ParArrayFromTo (sExp e1) (sExp e2)
+    ParArrayFromThenTo _ e1 e2 e3 -> S.ParArrayFromThenTo (sExp e1) (sExp e2) (sExp e3)
     ListComp _ e qss    -> S.ListComp (sExp e) (map sQualStmt qss)
     ParComp  _ e qsss   -> S.ParComp (sExp e) (map (map sQualStmt) qsss)
+    ParArrayComp  _ e qsss -> S.ParArrayComp (sExp e) (map (map sQualStmt) qsss)
     ExpTypeSig l e t    -> S.ExpTypeSig (getPointLoc l) (sExp e) (sType t)
     VarQuote _ qn       -> S.VarQuote (sQName qn)
     TypQuote _ qn       -> S.TypQuote (sQName qn)
@@ -379,6 +447,7 @@ sExp e = case e of
     RightArrApp     _ e1 e2 -> S.RightArrApp (sExp e1) (sExp e2)
     LeftArrHighApp  _ e1 e2 -> S.LeftArrHighApp (sExp e1) (sExp e2)
     RightArrHighApp _ e1 e2 -> S.RightArrHighApp (sExp e1) (sExp e2)
+    LCase _ alts -> S.LCase (map sAlt alts)
 
 
 sXName :: XName l -> S.XName
@@ -417,7 +486,13 @@ sModulePragma :: SrcInfo loc => ModulePragma loc -> S.ModulePragma
 sModulePragma pr = case pr of
     LanguagePragma   l ns   -> S.LanguagePragma (getPointLoc l) (map sName ns)
     OptionsPragma    l mt str -> S.OptionsPragma (getPointLoc l) mt str
-    AnnModulePragma  l ann -> S.AnnModulePragma (getPointLoc l) (sAnnotation ann)
+    AnnModulePragma  l ann' -> S.AnnModulePragma (getPointLoc l) (sAnnotation ann')
+
+sOverlap :: SrcInfo loc => Overlap loc -> S.Overlap
+sOverlap o' = case o' of
+    NoOverlap _   -> S.NoOverlap
+    Overlap _     -> S.Overlap
+    Incoherent _  -> S.Incoherent
 
 sActivation :: Activation l -> S.Activation
 sActivation act = case act of
@@ -428,7 +503,7 @@ sRule :: SrcInfo loc => Rule loc -> S.Rule
 sRule (Rule _ str mact mrvs e1 e2) =
     S.Rule str (maybe S.AlwaysActive sActivation mact) (fmap (map sRuleVar) mrvs) (sExp e1) (sExp e2)
 
-sRuleVar :: RuleVar l -> S.RuleVar
+sRuleVar :: SrcInfo l => RuleVar l -> S.RuleVar
 sRuleVar (RuleVar _ n) = S.RuleVar (sName n)
 sRuleVar (TypedRuleVar _ n t) = S.TypedRuleVar (sName n) (sType t)
 
@@ -439,8 +514,7 @@ sWarningText (WarnText _ str) = S.WarnText str
 sPat :: SrcInfo loc => Pat loc -> S.Pat
 sPat pat = case pat of
     PVar _ n            -> S.PVar (sName n)
-    PLit _ lit          -> S.PLit (sLiteral lit)
-    PNeg _ p            -> S.PNeg (sPat p)
+    PLit _ sg lit       -> S.PLit (sSign sg) (sLiteral lit)
     PNPlusK _ n k       -> S.PNPlusK (sName n) k
     PInfixApp _ pa qn pb -> S.PInfixApp (sPat pa) (sQName qn) (sPat pb)
     PApp _ qn ps        -> S.PApp (sQName qn) (map sPat ps)
@@ -459,7 +533,6 @@ sPat pat = case pat of
     PXPcdata _ str      -> S.PXPcdata str
     PXPatTag _ p        -> S.PXPatTag (sPat p)
     PXRPats  _ rps      -> S.PXRPats (map sRPat rps)
-    PExplTypeArg _ qn t -> S.PExplTypeArg (sQName qn) (sType t)
     PQuasiQuote _ nm qt -> S.PQuasiQuote nm qt
     PBangPat _ p        -> S.PBangPat (sPat p)
 
@@ -476,7 +549,7 @@ sRPatOp rpop = case rpop of
     RPOptG  _ -> S.RPOptG
 
 sRPat :: SrcInfo loc => RPat loc -> S.RPat
-sRPat rp = case rp of
+sRPat rp' = case rp' of
     RPOp _ rp rop       -> S.RPOp (sRPat rp) (sRPatOp rop)
     RPEither _ rp1 rp2  -> S.RPEither (sRPat rp1) (sRPat rp2)
     RPSeq _ rps         -> S.RPSeq (map sRPat rps)
@@ -489,7 +562,7 @@ sRPat rp = case rp of
 sPatField :: SrcInfo loc => PatField loc -> S.PatField
 sPatField pf = case pf of
     PFieldPat _ qn p    -> S.PFieldPat (sQName qn) (sPat p)
-    PFieldPun _ n       -> S.PFieldPun (sName n)
+    PFieldPun _ qn      -> S.PFieldPun (sQName qn)
     PFieldWildcard _    -> S.PFieldWildcard
 
 sStmt :: SrcInfo loc => Stmt loc -> S.Stmt
@@ -511,16 +584,8 @@ sQualStmt qs = case qs of
 sFieldUpdate :: SrcInfo loc => FieldUpdate loc -> S.FieldUpdate
 sFieldUpdate fu = case fu of
     FieldUpdate _ qn e      -> S.FieldUpdate (sQName qn) (sExp e)
-    FieldPun _ n            -> S.FieldPun (sName n)
+    FieldPun _ qn           -> S.FieldPun (sQName qn)
     FieldWildcard _         -> S.FieldWildcard
 
 sAlt :: SrcInfo loc => Alt loc -> S.Alt
-sAlt (Alt l p galts mbs) = S.Alt (getPointLoc l) (sPat p) (sGuardedAlts galts) (maybe (S.BDecls []) sBinds mbs)
-
-sGuardedAlts :: SrcInfo loc => GuardedAlts loc -> S.GuardedAlts
-sGuardedAlts galts = case galts of
-    UnGuardedAlt _ e    -> S.UnGuardedAlt (sExp e)
-    GuardedAlts  _ gs   -> S.GuardedAlts (map sGuardedAlt gs)
-
-sGuardedAlt :: SrcInfo loc => GuardedAlt loc -> S.GuardedAlt
-sGuardedAlt (GuardedAlt l ss e) = S.GuardedAlt (getPointLoc l) (map sStmt ss) (sExp e)
+sAlt (Alt l p galts mbs) = S.Alt (getPointLoc l) (sPat p) (sRhs galts) (maybe (S.BDecls []) sBinds mbs)
