@@ -1,3 +1,4 @@
+{-# LANGUAGE DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Exts.Extension
@@ -40,8 +41,9 @@ module Language.Haskell.Exts.Extension (
 
 import Control.Applicative ((<$>), (<|>))
 import Data.Array (Array, accumArray, bounds, Ix(inRange), (!))
-import Data.List (nub, (\\), delete)
+import Data.List (nub, delete)
 import Data.Maybe (fromMaybe)
+import Data.Data
 
 -- Copyright notice from Cabal's Language.Haskell.Extension,
 -- from which we borrow plenty of features:
@@ -93,15 +95,13 @@ data Language =
 
   -- | An unknown language, identified by its name.
   | UnknownLanguage String
-  deriving (Show, Read, Eq, Ord)
+  deriving (Show, Read, Eq, Ord, Data, Typeable)
 
 knownLanguages :: [Language]
 knownLanguages = [Haskell98, Haskell2010]
 
 classifyLanguage :: String -> Language
-classifyLanguage = \str -> case lookup str langTable of
-    Just lang -> lang
-    Nothing   -> UnknownLanguage str
+classifyLanguage str = fromMaybe (UnknownLanguage str) $ lookup str langTable
   where
     langTable = [ (show lang, lang)
                 | lang <- knownLanguages ]
@@ -145,14 +145,13 @@ data KnownExtension =
   -- instance being applicable.
   | IncoherentInstances
 
-{- DoRec not yet supported by HSE.
+  -- | [GHC &#xa7; 7.6.3.5] Allow type signatures in instances.
+  | InstanceSigs
 
   -- | [GHC &#xa7; 7.3.8] Allows recursive bindings in @do@ blocks,
   -- using the @rec@ keyword.
   | DoRec
 
--}
-
   -- | [GHC &#xa7; 7.3.8.2] Deprecated in GHC.  Allows recursive bindings
   -- using @mdo@, a variant of @do@.  @DoRec@ provides a different,
   -- preferred syntax.
@@ -282,6 +281,9 @@ data KnownExtension =
   -- identifiers.  Also enables literal syntax for unboxed values.
   | MagicHash
 
+  -- | Binary integer literals
+  | BinaryLiterals
+
   -- | [GHC &#xa7; 7.7] Allow data types and type synonyms which are
   -- indexed by types, i.e. ad-hoc polymorphism for types.
   | TypeFamilies
@@ -307,7 +309,8 @@ data KnownExtension =
   -- or type variable to be an infix operator.
   | TypeOperators
 
---PArr -- not ready yet, and will probably be renamed to ParallelArrays
+  -- | [GHC &#xa7; 7.26.5] Parallel arrays for Data Parallel Haskell.
+  | ParallelArrays
 
   -- | [GHC &#xa7; 7.3.16] Enable syntax for implicitly binding local names
   -- corresponding to the field names of a record.  A wildcard binds
@@ -365,6 +368,8 @@ data KnownExtension =
   -- > import "network" Network.Socket
   | PackageImports
 
+  | LambdaCase
+
   -- | [GHC &#xa7; 7.8.6] Deprecated in GHC 6.12 and will be removed in
   -- GHC 7.  Allow a type variable to be instantiated at a
   -- polymorphic type.
@@ -436,6 +441,9 @@ data KnownExtension =
   -- | Enable @deriving@ for the @Data.Functor.Functor@ class.
   | DeriveFunctor
 
+  -- | [GHC &#xa7; 7.5.3] Enable @deriving@ for the @Generic@ and @Generic1@ classes.
+  | DeriveGeneric
+
   -- | Enable @deriving@ for the @Data.Traversable.Traversable@ class.
   | DeriveTraversable
 
@@ -452,7 +460,16 @@ data KnownExtension =
   -- foreign function interface.
   | CApiFFI
 
- {- Safe Haskell not yet supported by HSE.
+  -- | [GHC &#xa7; 7.3.27] Enable explicit namespaces in import/export.
+  | ExplicitNamespaces
+
+  | DataKinds
+
+  | PolyKinds
+
+  -- | [GHC &#xa7; 7.3.16] Enable the multi-way if-expressions
+  -- extension to accept conditional expressions with multiple branches.
+  | MultiWayIf
 
   -- | [GHC &#xa7; 7.20.3] Allow imports to be qualified with a safe
   -- keyword that requires the imported module be trusted as according
@@ -472,9 +489,14 @@ data KnownExtension =
   -- trusted.
   | Trustworthy
 
--}
+  -- | [GHC &#xa7; 7.6.1.4] Allow default signatures in class
+  -- definitions. They apply to default methods implemented in this
+  -- class.
+  | DefaultSignatures
 
-{- ConstraintKinds not yet supported by HSE
+
+-- ConstraintKinds not generally supported by HSE. This flag makes RCategory2.hs
+-- pass testing though.
 
   -- | [GHC &#xa7; 7.40] Allow type class/implicit parameter/equality
   -- constraints to be used as types with the special kind Constraint.
@@ -482,9 +504,8 @@ data KnownExtension =
   -- Constraint can occur before the arrow.
   | ConstraintKinds
 
--}
 
-  deriving (Show, Read, Eq, Ord, Enum, Bounded)
+  deriving (Show, Read, Eq, Ord, Enum, Bounded, Data, Typeable)
 
 -- | Certain extensions imply other extensions, and this function
 --   makes the implication explicit. This also handles deprecated
@@ -516,10 +537,11 @@ impliesExts = go
                     PolymorphicComponents   -> [ExplicitForAll]
                     LiberalTypeSynonyms -> [ExplicitForAll]
                     ExistentialQuantification -> [ExplicitForAll]
+                    ImpredicativeTypes   -> [ExplicitForAll]
                     -- Deprecations
                     RecordPuns          -> [NamedFieldPuns]
                     PatternSignatures   -> [ScopedTypeVariables]
-                    e                   -> []
+                    _                   -> []
 
 -- | The list of extensions enabled by
 --   GHC's portmanteau -fglasgow-exts flag.
@@ -564,7 +586,7 @@ allLangDefault = [MonomorphismRestriction, MonoPatBinds, ImplicitPrelude]
 ghcDefault :: [Extension]
 ghcDefault = map EnableExtension (NondecreasingIndentation:allLangDefault)
 
--- | List of all known extensions, all enabled.
+-- | List of all known extensions, both \"yes\" and \"no\" versions.
 knownExtensions :: [Extension]
 knownExtensions =
   concat [ [EnableExtension x, DisableExtension x] | x <- [minBound..maxBound] ]
@@ -641,7 +663,7 @@ readMay s = case [x | (x,t) <- reads s, ("","") <- lex t] of
  -------------------------------------------}
 
 toExtensionList :: Language -> [Extension] -> [KnownExtension]
-toExtensionList lang exts =
+toExtensionList lang exts' =
     let langKes = case lang of
                     Haskell98 -> NPlusKPatterns:allLangDefault
                     Haskell2010 -> [DoAndIfThenElse
@@ -657,7 +679,7 @@ toExtensionList lang exts =
         remExts = [ ke | DisableExtension ke <- exts ]
      in impliesExts $ nub $ (langKes ++ addExts) \\ remExts
 -}
-  in impliesExts $ go langKes exts
+  in impliesExts $ go langKes exts'
     where go :: [KnownExtension] -> [Extension] -> [KnownExtension]
           go acc [] = acc
           go acc (DisableExtension x : exts) = go (nub (delete x acc)) exts
