@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Exts.Fixity
@@ -36,27 +36,16 @@ module Language.Haskell.Exts.Fixity
 
 import Language.Haskell.Exts.Syntax
 
-import Data.Char (isUpper)
-import Control.Monad (when, (<=<), liftM, liftM2, liftM3, liftM4)
+import Control.Monad (when, (<=<), liftM, liftM2, liftM3)
 import Data.Traversable (mapM)
 import Prelude hiding (mapM)
-
-#ifdef __GLASGOW_HASKELL__
-#ifdef BASE4
 import Data.Data hiding (Fixity)
-#else
-import Data.Generics (Data(..),Typeable(..))
-#endif
-#endif
+import Data.Maybe (fromMaybe)
 
 -- | Operator fixities are represented by their associativity
 --   (left, right or none) and their precedence (0-9).
 data Fixity = Fixity Assoc Int QName
-#ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
 
 -- | All AST elements that may include expressions which in turn may
 --   need fixity tweaking will be instances of this class.
@@ -70,14 +59,14 @@ class AppFixity ast where
 
 
 instance AppFixity Exp where
-  applyFixities fixs = infFix fixs <=< leafFix fixs
+  applyFixities fixs' = infFix fixs' <=< leafFix fixs'
     where -- This is the real meat case. We can assume a left-associative list to begin with.
           infFix fixs (InfixApp a op2 z) = do
               e <- infFix fixs a
               let fixup (a1,p1) (a2,p2) y pre = do
                       when (p1 == p2 && (a1 /= a2 || a1 == AssocNone)) -- Ambiguous infix expression!
                            $ fail "Ambiguous infix expression"
-                      if (p1 > p2 || p1 == p2 && (a1 == AssocLeft || a2 == AssocNone)) -- Already right order
+                      if p1 > p2 || p1 == p2 && (a1 == AssocLeft || a2 == AssocNone) -- Already right order
                        then return $ InfixApp e op2 z
                        else liftM pre (infFix fixs $ InfixApp y op2 z)
               case e of
@@ -88,19 +77,18 @@ instance AppFixity Exp where
           infFix _ e = return e
 
 instance AppFixity Pat where
-  applyFixities fixs = infFix fixs <=< leafFixP fixs
+  applyFixities fixs' = infFix fixs' <=< leafFixP fixs'
     where -- Same for patterns
           infFix fixs (PInfixApp a op2 z) = do
               p <- infFix fixs a
               let fixup (a1,p1) (a2,p2) y pre = do
                       when (p1 == p2 && (a1 /= a2 || a1 == AssocNone )) -- Ambiguous infix expression!
                            $ fail "Ambiguous infix expression"
-                      if (p1 > p2 || p1 == p2 && (a1 == AssocLeft || a2 == AssocNone)) -- Already right order
+                      if p1 > p2 || p1 == p2 && (a1 == AssocLeft || a2 == AssocNone) -- Already right order
                        then return $ PInfixApp p op2 z
                        else liftM pre (infFix fixs $ PInfixApp y op2 z)
               case p of
                PInfixApp x op1 y -> fixup (askFixityP fixs op1) (askFixityP fixs op2) y (PInfixApp x op1)
-               PNeg            y -> fixup prefixMinusFixity     (askFixityP fixs op2) y  PNeg
                _  -> return $ PInfixApp p op2 z
 
           infFix _ p = return p
@@ -126,9 +114,7 @@ askFixityP xs qn = askFix xs (g qn)
 askFix :: [Fixity] -> QName -> (Assoc, Int)
 askFix xs = \k -> lookupWithDefault (AssocLeft, 9) k mp
     where
-        lookupWithDefault def k mp = case lookup k mp of
-            Nothing -> def
-            Just x  -> x
+        lookupWithDefault def k mp1 = fromMaybe def $ lookup k mp1
 
         mp = [(x,(a,p)) | Fixity a p x <- xs]
 
@@ -206,10 +192,10 @@ instance AppFixity Module where
 instance AppFixity Decl where
     applyFixities fixs decl = case decl of
         ClassDecl loc ctxt n vars deps cdecls   -> liftM (ClassDecl loc ctxt n vars deps) $ mapM fix cdecls
-        InstDecl loc ctxt n ts idecls           -> liftM (InstDecl loc ctxt n ts) $ mapM fix idecls
+        InstDecl loc olp tvs ctxt n ts idecls   -> liftM (InstDecl loc olp tvs ctxt n ts) $ mapM fix idecls
         SpliceDecl loc spl      -> liftM (SpliceDecl loc) $ fix spl
         FunBind matches         -> liftM FunBind $ mapM fix matches
-        PatBind loc p mt rhs bs -> liftM3 (flip (PatBind loc) mt) (fix p) (fix rhs) (fix bs)
+        PatBind loc p rhs bs    -> liftM3 (PatBind loc) (fix p) (fix rhs) (fix bs)
         AnnPragma loc ann       -> liftM (AnnPragma loc) $ fix ann
         _                       -> return decl
       where fix x = applyFixities fixs x
@@ -258,7 +244,7 @@ instance AppFixity PatField where
     applyFixities _ pf = return pf
 
 instance AppFixity RPat where
-    applyFixities fixs rp = case rp of
+    applyFixities fixs rp' = case rp' of
         RPOp rp op          -> liftM (flip RPOp op) (fix rp)
         RPEither a b        -> liftM2 RPEither (fix a) (fix b)
         RPSeq rps           -> liftM RPSeq $ mapM fix rps
@@ -298,16 +284,6 @@ instance AppFixity Alt where
     applyFixities fixs (Alt loc p galts bs) = liftM3 (Alt loc) (fix p) (fix galts) (fix bs)
       where fix x = applyFixities fixs x
 
-instance AppFixity GuardedAlts where
-    applyFixities fixs galts = case galts of
-        UnGuardedAlt e      -> liftM UnGuardedAlt $ fix e
-        GuardedAlts  galts  -> liftM GuardedAlts $ mapM fix galts
-      where fix x = applyFixities fixs x
-
-instance AppFixity GuardedAlt where
-    applyFixities fixs (GuardedAlt loc stmts e) = liftM2 (GuardedAlt loc) (mapM fix stmts) (fix e)
-      where fix x = applyFixities fixs x
-
 instance AppFixity QualStmt where
     applyFixities fixs qstmt = case qstmt of
         QualStmt     s      -> liftM QualStmt $ fix s
@@ -338,18 +314,21 @@ instance AppFixity XAttr where
 -- Recursively fixes the "leaves" of the infix chains,
 -- without yet touching the chain itself. We assume all chains are
 -- left-associate to begin with.
-leafFix fixs e = case e of
+leafFix :: Monad m => [Fixity] -> Exp -> m Exp
+leafFix fixs e' = case e' of
     InfixApp e1 op e2       -> liftM2 (flip InfixApp op) (leafFix fixs e1) (fix e2)
     App e1 e2               -> liftM2 App (fix e1) (fix e2)
     NegApp e                -> liftM NegApp $ fix e
     Lambda loc pats e       -> liftM2 (Lambda loc) (mapM fix pats) $ fix e
     Let bs e                -> liftM2 Let (fix bs) $ fix e
     If e a b                -> liftM3 If (fix e) (fix a) (fix b)
+    MultiIf alts            -> liftM MultiIf (mapM fix alts)
     Case e alts             -> liftM2 Case (fix e) $ mapM fix alts
     Do stmts                -> liftM Do $ mapM fix stmts
     MDo stmts               -> liftM MDo $ mapM fix stmts
     Tuple bx exps           -> liftM (Tuple bx) $ mapM fix exps
     List exps               -> liftM List $ mapM fix  exps
+    ParArray exps           -> liftM ParArray $ mapM fix exps
     Paren e                 -> liftM Paren $ fix e
     LeftSection e op        -> liftM (flip LeftSection op) (fix e)
     RightSection op e       -> liftM (RightSection op) $ fix e
@@ -359,8 +338,11 @@ leafFix fixs e = case e of
     EnumFromTo e1 e2        -> liftM2 EnumFromTo (fix e1) (fix e2)
     EnumFromThen e1 e2      -> liftM2 EnumFromThen (fix e1) (fix e2)
     EnumFromThenTo e1 e2 e3 -> liftM3 EnumFromThenTo (fix e1) (fix e2) (fix e3)
+    ParArrayFromTo e1 e2     -> liftM2 ParArrayFromTo (fix e1) (fix e2)
+    ParArrayFromThenTo e1 e2 e3 -> liftM3 ParArrayFromThenTo (fix e1) (fix e2) (fix e3)
     ListComp e quals        -> liftM2 ListComp (fix e) $ mapM fix quals
     ParComp  e qualss       -> liftM2 ParComp (fix e) $ mapM (mapM fix) qualss
+    ParArrayComp  e qualss  -> liftM2 ParArrayComp (fix e) $ mapM (mapM fix) qualss
     ExpTypeSig loc e t      -> liftM (flip (ExpTypeSig loc) t) (fix e)
     BracketExp b            -> liftM BracketExp $ fix b
     SpliceExp s             -> liftM SpliceExp $ fix s
@@ -376,14 +358,15 @@ leafFix fixs e = case e of
     CorePragma s e          -> liftM (CorePragma s) (fix e)
     SCCPragma s e           -> liftM (SCCPragma s) (fix e)
     GenPragma s ab cd e     -> liftM (GenPragma s ab cd) (fix e)
+    LCase alts              -> liftM LCase $ mapM fix alts
 
-    _                       -> return e
+    _                       -> return e'
   where
     fix x = applyFixities fixs x
 
-leafFixP fixs p = case p of
+leafFixP :: Monad m => [Fixity] -> Pat -> m Pat
+leafFixP fixs p' = case p' of
         PInfixApp p1 op p2    -> liftM2 (flip PInfixApp op) (leafFixP fixs p1) (fix p2)
-        PNeg p                -> liftM PNeg $ fix p
         PApp n ps             -> liftM (PApp n) $ mapM fix ps
         PTuple bx ps          -> liftM (PTuple bx) $ mapM fix ps
         PList ps              -> liftM PList $ mapM fix ps
@@ -399,5 +382,5 @@ leafFixP fixs p = case p of
         PXPatTag p            -> liftM PXPatTag $ fix p
         PXRPats rps           -> liftM PXRPats $ mapM fix rps
         PBangPat p            -> liftM PBangPat $ fix p
-        _                     -> return p
+        _                     -> return p'
       where fix x = applyFixities fixs x
