@@ -21,14 +21,15 @@
 module Language.Haskell.Exts.InternalLexer (Token(..), showToken, lexer, topLexer) where
 
 import Language.Haskell.Exts.ParseMonad
-import Language.Haskell.Exts.SrcLoc
+import Language.Haskell.Exts.SrcLoc hiding (loc)
 import Language.Haskell.Exts.Comments
 import Language.Haskell.Exts.Extension
 import Language.Haskell.Exts.ExtScheme
 
+import Prelude hiding (id, exponent)
 import Data.Char
 import Data.Ratio
-import Data.List (intersperse)
+import Data.List (intercalate, isPrefixOf)
 import Control.Monad (when)
 
 -- import Debug.Trace (trace)
@@ -62,14 +63,14 @@ data Token
         | RightParen
         | LeftHashParen
         | RightHashParen
-        | LeftCurlyBar
-        | RightCurlyBar
         | SemiColon
         | LeftCurly
         | RightCurly
         | VRightCurly           -- a virtual close brace
         | LeftSquare
         | RightSquare
+        | ParArrayLeftSquare -- [:
+        | ParArrayRightSquare -- :]
         | Comma
         | Underscore
         | BackQuote
@@ -79,6 +80,7 @@ data Token
         | Dot           -- reserved for use with 'forall x . x'
         | DotDot
         | Colon
+        | QuoteColon
         | DoubleColon
         | Equals
         | Backslash
@@ -145,6 +147,10 @@ data Token
 --        | INCLUDE String
         | LANGUAGE
         | ANN
+        | MINIMAL
+        | NO_OVERLAP
+        | OVERLAP
+        | INCOHERENT
 
 -- Reserved Ids
 
@@ -214,6 +220,9 @@ reserved_ops = [
  ( "~",  (Tilde,        Nothing) ),
  ( "=>", (DoubleArrow,  Nothing) ),
  ( "*",  (Star,         Just (Any [KindSignatures])) ),
+ -- Parallel arrays
+ ( "[:", (ParArrayLeftSquare,   Just (Any [ParallelArrays])) ),
+ ( ":]", (ParArrayRightSquare,  Just (Any [ParallelArrays])) ),
  -- Arrows notation
  ( "-<",  (LeftArrowTail,       Just (Any [Arrows])) ),
  ( ">-",  (RightArrowTail,      Just (Any [Arrows])) ),
@@ -228,7 +237,8 @@ reserved_ops = [
  ( "\x291a",    (RightArrowTail,    Just (All [UnicodeSyntax, Arrows])) ),
  ( "\x291b",    (LeftDblArrowTail,  Just (All [UnicodeSyntax, Arrows])) ),
  ( "\x291c",    (RightDblArrowTail, Just (All [UnicodeSyntax, Arrows])) ),
- ( "\x2605",    (Star,              Just (All [UnicodeSyntax, KindSignatures])) )
+ ( "\x2605",    (Star,              Just (All [UnicodeSyntax, KindSignatures])) ),
+ ( "\x2200",    (KW_Forall,         Just (All [UnicodeSyntax, ExplicitForAll])) )
  ]
 
 special_varops :: [(String,(Token, Maybe ExtScheme))]
@@ -266,17 +276,14 @@ reserved_ids = [
  ( "newtype",   (KW_NewType,    Nothing) ),
  ( "of",        (KW_Of,         Nothing) ),
  ( "proc",      (KW_Proc,       Just (Any [Arrows])) ),
- ( "rec",       (KW_Rec,        Just (Any [Arrows])) ),
+ ( "rec",       (KW_Rec,        Just (Any [Arrows, RecursiveDo, DoRec])) ),
  ( "then",      (KW_Then,       Nothing) ),
  ( "type",      (KW_Type,       Nothing) ),
  ( "using",     (KW_Using,      Just (Any [TransformListComp])) ),
  ( "where",     (KW_Where,      Nothing) ),
 
 -- FFI
- ( "foreign",   (KW_Foreign,    Just (Any [ForeignFunctionInterface])) ),
-
--- Unicode
- ( "\x2200",    (KW_Forall,     Just (All [UnicodeSyntax, ExplicitForAll])) )
+ ( "foreign",   (KW_Foreign,    Just (Any [ForeignFunctionInterface])) )
  ]
 
 
@@ -288,7 +295,7 @@ special_varids = [
 
 -- FFI
  ( "export",        (KW_Export,        Just (Any [ForeignFunctionInterface])) ),
- ( "safe",          (KW_Safe,          Just (Any [ForeignFunctionInterface])) ),
+ ( "safe",          (KW_Safe,          Just (Any [ForeignFunctionInterface, SafeImports, Safe, Trustworthy])) ),
  ( "unsafe",        (KW_Unsafe,        Just (Any [ForeignFunctionInterface])) ),
  ( "threadsafe",    (KW_Threadsafe,    Just (Any [ForeignFunctionInterface])) ),
  ( "interruptible", (KW_Interruptible, Just (Any [InterruptibleFFI])) ),
@@ -318,16 +325,27 @@ pragmas = [
  ( "core",              CORE            ),
  ( "unpack",            UNPACK          ),
  ( "language",          LANGUAGE        ),
+ ( "minimal",           MINIMAL         ),
+ ( "no_overlap",        NO_OVERLAP      ),
+ ( "overlap",           OVERLAP         ),
+ ( "incoherent",        INCOHERENT      ),
  ( "options",           OPTIONS undefined ) -- we'll tweak it before use - promise!
 -- ( "cfiles",            CFILES  undefined ), -- same here...
 -- ( "include",           INCLUDE undefined )  -- ...and here!
  ]
 
-isIdent, isHSymbol :: Char -> Bool
+isIdent, isHSymbol, isPragmaChar :: Char -> Bool
 isIdent   c = isAlpha c || isDigit c || c == '\'' || c == '_'
 
 isHSymbol c = c `elem` ":!#%&*./?@\\-" || ((isSymbol c || isPunctuation c) && not (c `elem` "(),;[]`{}_\"'"))
 
+isPragmaChar c = isAlphaNum c || c == '_'
+
+-- | Checks whether the character would be legal in some position of a qvar.
+--   Means that '..' and "AAA" will pass the test.
+isPossiblyQvar :: Char -> Bool
+isPossiblyQvar c = isIdent (toLower c) || c == '.'
+
 matchChar :: Char -> String -> Lex a ()
 matchChar c msg = do
     s <- getInput
@@ -348,7 +366,7 @@ topLexer = do
               setBOL >> getSrcLocL >>= \l -> return (Loc (mkSrcSpan l l) VRightCurly)
      else do
         bol <- checkBOL
-        (bol, ws) <- lexWhiteSpace bol
+        (bol', ws) <- lexWhiteSpace bol
         -- take care of whitespace in PCDATA
         ec <- getExtContext
         case ec of
@@ -356,11 +374,11 @@ topLexer = do
          -- then we want to care about the whitespace.
          -- We don't bother to test for XmlSyntax, since we
          -- couldn't end up in ChildCtxt otherwise.
-         Just ChildCtxt | not bol && ws -> getSrcLocL >>= \l -> return $ Loc (mkSrcSpan l l) $ XPCDATA " "
+         Just ChildCtxt | not bol' && ws -> getSrcLocL >>= \l -> return $ Loc (mkSrcSpan l l) $ XPCDATA " "
          _ -> do startToken
                  sl <- getSrcLocL
-                 t <- if bol then lexBOL    -- >>= \t -> trace ("BOL: " ++ show t) (return t)
-                             else lexToken  -- >>= \t -> trace (show t) (return t)
+                 t <- if bol' then lexBOL    -- >>= \t -> trace ("BOL: " ++ show t) (return t)
+                              else lexToken  -- >>= \t -> trace (show t) (return t)
                  el <- getSrcLocL
                  return $ Loc (mkSrcSpan sl el) t
 
@@ -379,12 +397,12 @@ lexWhiteSpace bol = do
         '{':'-':_ -> do
             loc <- getSrcLocL
             discard 2
-            (bol, c) <- lexNestedComment bol ""
+            (bol1, c) <- lexNestedComment bol ""
             loc2 <- getSrcLocL
             pushComment $ Comment True (mkSrcSpan loc loc2) (reverse c)
-            (bol, _) <- lexWhiteSpace bol
-            return (bol, True)
-        '-':'-':s | all (== '-') (takeWhile isHSymbol s) -> do
+            (bol2, _) <- lexWhiteSpace bol1
+            return (bol2, True)
+        '-':'-':s1 | all (== '-') (takeWhile isHSymbol s1) -> do
             loc    <- getSrcLocL
             discard 2
             dashes <- lexWhile (== '-')
@@ -397,25 +415,30 @@ lexWhiteSpace bol = do
                 _ -> do
                     pushComment com
                     lexNewline
-                    lexWhiteSpace True
+                    lexWhiteSpace_ True
                     return (True, True)
         '\n':_ -> do
             lexNewline
-            lexWhiteSpace True
+            lexWhiteSpace_ True
             return (True, True)
         '\t':_ -> do
             lexTab
-            (bol, _) <- lexWhiteSpace bol
-            return (bol, True)
+            (bol', _) <- lexWhiteSpace bol
+            return (bol', True)
         c:_ | isSpace c -> do
             discard 1
-            (bol, _) <- lexWhiteSpace bol
-            return (bol, True)
+            (bol', _) <- lexWhiteSpace bol
+            return (bol', True)
         _ -> return (bol, False)
 
+-- | lexWhiteSpace without the return value.
+lexWhiteSpace_ :: Bool -> Lex a ()
+lexWhiteSpace_ bol =  do _ <- lexWhiteSpace bol
+                         return ()
+
 isRecognisedPragma, isLinePragma :: String -> Bool
-isRecognisedPragma str = let pragma = map toLower . takeWhile isAlphaNum . dropWhile isSpace $ str
-                          in case lookup pragma pragmas of
+isRecognisedPragma str = let pragma = takeWhile isPragmaChar . dropWhile isSpace $ str
+                          in case lookupKnownPragma pragma of
                               Nothing -> False
                               _       -> True
 
@@ -427,17 +450,17 @@ isLinePragma str = let pragma = map toLower . takeWhile isAlphaNum . dropWhile i
 lexLinePragma :: Lex a (Int, String)
 lexLinePragma = do
     discard 3   -- {-#
-    lexWhile isSpace
+    lexWhile_ isSpace
     discard 4   -- LINE
-    lexWhile isSpace
+    lexWhile_ isSpace
     i <- lexWhile isDigit
     when (null i) $ fail "Improperly formatted LINE pragma"
-    lexWhile isSpace
+    lexWhile_ isSpace
     matchChar '"' "Improperly formatted LINE pragma"
     fn <- lexWhile (/= '"')
     matchChar '"' "Impossible - lexLinePragma"
-    lexWhile isSpace
-    mapM (flip matchChar "Improperly formatted LINE pragma") "#-}"
+    lexWhile_ isSpace
+    mapM_ (flip matchChar "Improperly formatted LINE pragma") "#-}"
     lexNewline
     return (read i, fn)
 
@@ -448,8 +471,8 @@ lexNestedComment bol str = do
         '-':'}':_ -> discard 2 >> return (bol, str)
         '{':'-':_ -> do
             discard 2
-            (bol, c) <- lexNestedComment bol ("-{" ++ str) -- rest of the subcomment
-            lexNestedComment bol ("}-" ++ c  ) -- rest of this comment
+            (bol', c) <- lexNestedComment bol ("-{" ++ str) -- rest of the subcomment
+            lexNestedComment bol' ("}-" ++ c  ) -- rest of this comment
         '\t':_    -> lexTab >> lexNestedComment bol ('\t':str)
         '\n':_    -> lexNewline >> lexNestedComment True ('\n':str)
         c:_       -> discard 1 >> lexNestedComment bol (c:str)
@@ -528,14 +551,16 @@ lexPCDATA = do
                 case x of
                  XPCDATA p -> return $ XPCDATA $ '\n':p
                  EOF -> return EOF
+                 _ -> fail $ "lexPCDATA: unexpected token: " ++ show x
             '<':_ -> return $ XPCDATA ""
-            _ -> do let pcd = takeWhile (\c -> not $ elem c "<\n") s
+            _ -> do let pcd = takeWhile (\c -> c `notElem` "<\n") s
                         l = length pcd
                     discard l
                     x <- lexPCDATA
                     case x of
                      XPCDATA pcd' -> return $ XPCDATA $ pcd ++ pcd'
                      EOF -> return EOF
+                     _ -> fail $ "lexPCDATA: unexpected token: " ++ show x
 
 
 lexCodeTagCtxt :: Lex a Token
@@ -598,6 +623,11 @@ lexStdToken = do
                         (n, str) <- lexOctal
                         con <- intHash
                         return (con (n, '0':c:str))
+                  | toLower c == 'b' && isBinDigit d && BinaryLiterals `elem` exts -> do
+                        discard 2
+                        (n, str) <- lexBinary
+                        con <- intHash
+                        return (con (n, '0':c:str))
                   | toLower c == 'x' && isHexDigit d -> do
                         discard 2
                         (n, str) <- lexHexadecimal
@@ -632,11 +662,11 @@ lexStdToken = do
         -- template haskell
         '[':'|':_ | TemplateHaskell `elem` exts -> do
                 discard 2
-                return $ THExpQuote
+                return THExpQuote
 
         '[':c:'|':_ | c == 'e' && TemplateHaskell `elem` exts -> do
                         discard 3
-                        return $ THExpQuote
+                        return THExpQuote
                     | c == 'p' && TemplateHaskell `elem` exts -> do
                         discard 3
                         return THPatQuote
@@ -647,10 +677,12 @@ lexStdToken = do
                         discard 3
                         return THTypQuote
         '[':'$':c:_ | isLower c && QuasiQuotes `elem` exts ->
-                        discard 2 >> lexQuasiQuote
+                        discard 2 >> lexQuasiQuote c
 
-        '[':c:s | isLower c && QuasiQuotes `elem` exts && case dropWhile isIdent s of { '|':_ -> True;_->False} ->
-                        discard 1 >> lexQuasiQuote
+        '[':c:s' | isLower c && QuasiQuotes `elem` exts && case dropWhile isIdent s' of { '|':_ -> True;_->False} ->
+                        discard 1 >> lexQuasiQuote c
+                 | isUpper c && QuasiQuotes `elem` exts && case dropWhile isPossiblyQvar s' of { '|':_ -> True;_->False} ->
+                        discard 1 >> lexQuasiQuote c
 
         '|':']':_ | TemplateHaskell `elem` exts -> do
                         discard 2
@@ -666,7 +698,7 @@ lexStdToken = do
         -- end template haskell
 
         -- hsx
-        '<':'%':c:_ | XmlSyntax `elem` exts -> do
+        '<':'%':c:_ | XmlSyntax `elem` exts ->
                         case c of
                          '>' -> do discard 3
                                    pushExtContextL ChildCtxt
@@ -680,19 +712,21 @@ lexStdToken = do
                         return XStdTagOpen
         -- end hsx
 
-        '(':'#':c:_ | UnboxedTuples `elem` exts && not (isHSymbol c) -> do discard 2 >> return LeftHashParen
+        '(':'#':c:_ | UnboxedTuples `elem` exts && not (isHSymbol c) -> discard 2 >> return LeftHashParen
 
-        '#':')':_ | UnboxedTuples `elem` exts -> do discard 2 >> return RightHashParen
-
-        '{':'|':_ | Generics `elem` exts -> do discard 2 >> return LeftCurlyBar
-
-        '|':'}':_ | Generics `elem` exts -> do discard 2 >> return RightCurlyBar
+        '#':')':_ | UnboxedTuples `elem` exts -> discard 2 >> return RightHashParen
 
         -- pragmas
 
-        '{':'-':'#':_ -> do discard 3 >> lexPragmaStart
+        '{':'-':'#':_ -> saveExtensionsL >> discard 3 >> lexPragmaStart
 
-        '#':'-':'}':_ -> do discard 3 >> return PragmaEnd
+        '#':'-':'}':_ -> restoreExtensionsL >> discard 3 >> return PragmaEnd
+
+        -- Parallel arrays
+
+        '[':':':_ | ParallelArrays `elem` exts -> discard 2 >> return ParArrayLeftSquare
+
+        ':':']':_ | ParallelArrays `elem` exts -> discard 2 >> return ParArrayRightSquare
 
         c:_ | isDigit c -> lexDecimalOrFloat
 
@@ -702,7 +736,7 @@ lexStdToken = do
                     idents <- lexIdents
                     case idents of
                      [ident] -> case lookup ident (reserved_ids ++ special_varids) of
-                                 Just (keyword, scheme) -> do
+                                 Just (keyword, scheme) ->
                                     -- check if an extension keyword is enabled
                                     if isEnabled scheme exts
                                      then flagKW keyword >> return keyword
@@ -765,13 +799,21 @@ lexStdToken = do
                         return [ident ++ "#"]
                  _ -> return [ident]
 
-            lexQuasiQuote :: Lex a Token
-            lexQuasiQuote = do
+            lexQuasiQuote :: Char -> Lex a Token
+            lexQuasiQuote c = do
                 -- We've seen and dropped [$ already
-                ident <- lexWhile isIdent
+                ident <- lexQuoter
                 matchChar '|' "Malformed quasi-quote quoter"
                 body <- lexQQBody
                 return $ THQuasiQuote (ident, body)
+                  where lexQuoter
+                         | isLower c = lexWhile isIdent
+                         | otherwise = do
+                            qualThing <- lexConIdOrQual ""
+                            case qualThing of
+                                QVarId (s1,s2) -> return $ s1 ++ '.':s2
+                                QVarSym (s1, s2) -> return $ s1 ++ '.':s2
+                                _                -> fail "Malformed quasi-quote quoter"
 
             lexQQBody :: Lex a String
             lexQQBody = do
@@ -801,47 +843,61 @@ lexStdToken = do
                           rest <- lexQQBody
                           return (str++rest)
 
+-- Underscores are used in some pragmas. Options pragmas are a special case
+-- with our representation: the thing after the underscore is a parameter.
+-- Strip off the parameters to option pragmas by hand here, everything else
+-- sits in the pragmas map.
+lookupKnownPragma :: String -> Maybe Token
+lookupKnownPragma s =
+    case map toLower s of
+      x | "options_" `isPrefixOf` x -> Just $ OPTIONS (Just $ drop 8 s, undefined)
+        | "options" == x            -> Just $ OPTIONS (Nothing, undefined)
+        | otherwise                 -> lookup x pragmas
+
 lexPragmaStart :: Lex a Token
 lexPragmaStart = do
-    lexWhile isSpace
-    pr <- lexWhile isAlphaNum
-    case lookup (map toLower pr) pragmas of
+    lexWhile_ isSpace
+    pr <- lexWhile isPragmaChar
+    case lookupKnownPragma pr of
      Just (INLINE True) -> do
             s <- getInput
             case map toLower s of
-             '_':'c':'o':'n':'l':'i':'k':'e':_  -> do
+             ' ':'c':'o':'n':'l':'i':'k':'e':_  -> do
                       discard 8
-                      return $ INLINE_CONLIKE
+                      return INLINE_CONLIKE
              _ -> return $ INLINE True
      Just SPECIALISE -> do
             s <- getInput
             case dropWhile isSpace $ map toLower s of
              'i':'n':'l':'i':'n':'e':_ -> do
-                      lexWhile isSpace
+                      lexWhile_ isSpace
                       discard 6
                       return $ SPECIALISE_INLINE True
              'n':'o':'i':'n':'l':'i':'n':'e':_ -> do
-                        lexWhile isSpace
+                        lexWhile_ isSpace
                         discard 8
                         return $ SPECIALISE_INLINE False
              'n':'o':'t':'i':'n':'l':'i':'n':'e':_ -> do
-                        lexWhile isSpace
+                        lexWhile_ isSpace
                         discard 9
                         return $ SPECIALISE_INLINE False
              _ -> return SPECIALISE
 
-     Just (OPTIONS _) -> do     -- see, I promised we'd mask out the 'undefined'
-            s <- getInput
-            case s of
-             '_':_  -> do
-                discard 1
-                com <- lexWhile isIdent
+     Just (OPTIONS opt) ->     -- see, I promised we'd mask out the 'undefined'
+            case fst opt of
+             Just opt' -> do
                 rest <- lexRawPragma
-                return $ OPTIONS (Just com, rest)
-             x:_ | isSpace x -> do
-                rest <- lexRawPragma
-                return $ OPTIONS (Nothing, rest)
-             _ -> fail "Malformed Options pragma"
+                return $ OPTIONS (Just opt', rest)
+             Nothing -> do
+                            s <- getInput
+                            case s of
+                                x:_ | isSpace x -> do
+                                    rest <- lexRawPragma
+                                    return $ OPTIONS (Nothing, rest)
+                                _  -> fail "Malformed Options pragma"
+     Just RULES -> do -- Rules enable ScopedTypeVariables locally.
+            addExtensionL ScopedTypeVariables
+            return RULES
 {-     Just (CFILES _) -> do
             rest <- lexRawPragma
             return $ CFILES rest
@@ -857,9 +913,7 @@ lexPragmaStart = do
                   -- topLexer -- we just discard it as a comment for now and restart -}
 
 lexRawPragma :: Lex a String
-lexRawPragma = do
-    rpr <- lexRawPragmaAux
-    return $ dropWhile isSpace rpr
+lexRawPragma = lexRawPragmaAux
  where lexRawPragmaAux = do
         rpr <- lexWhile (/='#')
         s <- getInput
@@ -923,7 +977,7 @@ lexHash a b c = do
         r <- getInput
         case r of
          '#':'#':_ -> case c of
-                       Right c -> discard 2 >> return c
+                       Right c' -> discard 2 >> return c'
                        Left s  -> fail s
          '#':_     -> discard 1 >> return b
          _         ->              return a
@@ -945,13 +999,13 @@ lexConIdOrQual qual = do
                     discard 1
                     ident <- lexWhile isIdent
                     s <- getInput
-                    exts <- getExtensionsL
+                    exts' <- getExtensionsL
                     ident' <- case s of
-                               '#':_ | MagicHash `elem` exts -> discard 1 >> return (ident ++ "#")
+                               '#':_ | MagicHash `elem` exts' -> discard 1 >> return (ident ++ "#")
                                _ -> return ident
                     case lookup ident' reserved_ids of
                        -- cannot qualify a reserved word
-                       Just (_,scheme) | isEnabled scheme exts  -> just_a_conid
+                       Just (_,scheme) | isEnabled scheme exts'  -> just_a_conid
                        _ -> return (QVarId (qual', ident'))
 
              | isUpper c -> do      -- qualified conid?
@@ -961,10 +1015,10 @@ lexConIdOrQual qual = do
              | isHSymbol c -> do    -- qualified symbol?
                     discard 1
                     sym <- lexWhile isHSymbol
-                    exts <- getExtensionsL
+                    exts' <- getExtensionsL
                     case lookup sym reserved_ops of
                         -- cannot qualify a reserved operator
-                        Just (_,scheme) | isEnabled scheme exts -> just_a_conid
+                        Just (_,scheme) | isEnabled scheme exts' -> just_a_conid
                         _        -> return $ case c of
                                               ':' -> QConSym (qual', sym)
                                               _   -> QVarSym (qual', sym)
@@ -975,8 +1029,9 @@ lexConIdOrQual qual = do
               not (isIdent $ head cs) && MagicHash `elem` exts -> do
                 discard 1
                 case conid of
-                 ConId con -> return $ ConId $ con ++ "#"
-                 QConId (q,con) -> return $ QConId (q,con ++ "#")
+                 ConId con' -> return $ ConId $ con' ++ "#"
+                 QConId (q,con') -> return $ QConId (q,con' ++ "#")
+                 _ -> fail $ "lexConIdOrQual: unexpected token: " ++ show conid
           _ ->  return conid -- not a qualified thing
 
 lexCharacter :: Lex a Token
@@ -997,7 +1052,7 @@ lexCharacter = do   -- We need to keep track of not only character constants but
                     con <- lexHash Character CharacterHash
                             (Left "Double hash not available for character literals")
                     return (con (c, [c]))
-         _ | TemplateHaskell `elem` exts -> return THVarQuote
+         _ | any (`elem` exts) [TemplateHaskell, DataKinds] -> return THVarQuote
          _ -> fail "Improper character constant or misplaced \'"
 
     where matchQuote = matchChar '\'' "Improperly terminated character constant"
@@ -1112,17 +1167,17 @@ lexEscape = do
         'o':c:_ | isOctDigit c -> do
                     discard 1
                     (n, raw) <- lexOctal
-                    n <- checkChar n
-                    return (n, 'o':raw)
+                    n' <- checkChar n
+                    return (n', 'o':raw)
         'x':c:_ | isHexDigit c -> do
                     discard 1
                     (n, raw) <- lexHexadecimal
-                    n <- checkChar n
-                    return (n, 'x':raw)
+                    n' <- checkChar n
+                    return (n', 'x':raw)
         c:_ | isDigit c -> do
                     (n, raw) <- lexDecimal
-                    n <- checkChar n
-                    return (n, raw)
+                    n' <- checkChar n
+                    return (n', raw)
 
         _       -> fail "Illegal escape sequence"
 
@@ -1142,6 +1197,12 @@ lexOctal = do
     ds <- lexWhile isOctDigit
     return (parseInteger 8 ds, ds)
 
+-- assumes at least one binary digit
+lexBinary :: Lex a (Integer, String)
+lexBinary = do
+    ds <- lexWhile isBinDigit
+    return (parseInteger 2 ds, ds)
+
 -- assumes at least one hexadecimal digit
 lexHexadecimal :: Lex a (Integer, String)
 lexHexadecimal = do
@@ -1160,12 +1221,14 @@ parseInteger radix ds =
     foldl1 (\n d -> n * radix + d) (map (toInteger . digitToInt) ds)
 
 flagKW :: Token -> Lex a ()
-flagKW t = do
+flagKW t =
   when (t `elem` [KW_Do, KW_MDo]) $ do
        exts <- getExtensionsL
-       when (NondecreasingIndentation `elem` exts) $
-            flagDo
+       when (NondecreasingIndentation `elem` exts) flagDo
 
+-- | Selects ASCII binary digits, i.e. @\'0\'@..@\'1\'@.
+isBinDigit :: Char -> Bool
+isBinDigit c =  c >= '0' && c <= '1'
 ------------------------------------------------------------------
 -- "Pretty" printing for tokens
 
@@ -1177,7 +1240,7 @@ showToken t = case t of
   ILinVarId s       -> '%':s
   ConId s           -> s
   QConId (q,s)      -> q ++ '.':s
-  DVarId ss         -> concat $ intersperse "-" ss
+  DVarId ss         -> intercalate "-" ss
   VarSym s          -> s
   ConSym s          -> s
   QVarSym (q,s)     -> q ++ '.':s
@@ -1196,17 +1259,18 @@ showToken t = case t of
   RightParen        -> ")"
   LeftHashParen     -> "(#"
   RightHashParen    -> "#)"
-  LeftCurlyBar      -> "{|"
-  RightCurlyBar     -> "|}"
   SemiColon         -> ";"
   LeftCurly         -> "{"
   RightCurly        -> "}"
   VRightCurly       -> "virtual }"
   LeftSquare        -> "["
   RightSquare       -> "]"
+  ParArrayLeftSquare -> "[:"
+  ParArrayRightSquare -> ":]"
   Comma             -> ","
   Underscore        -> "_"
   BackQuote         -> "`"
+  QuoteColon        -> "':"
   Dot               -> "."
   DotDot            -> ".."
   Colon             -> ":"
@@ -1251,7 +1315,7 @@ showToken t = case t of
   PragmaEnd         -> "#-}"
   RULES             -> "{-# RULES"
   INLINE b          -> "{-# " ++ if b then "INLINE" else "NOINLINE"
-  INLINE_CONLIKE    -> "{-# " ++ "INLINE_CONLIKE"
+  INLINE_CONLIKE    -> "{-# " ++ "INLINE CONLIKE"
   SPECIALISE        -> "{-# SPECIALISE"
   SPECIALISE_INLINE b -> "{-# SPECIALISE " ++ if b then "INLINE" else "NOINLINE"
   SOURCE            -> "{-# SOURCE"
@@ -1261,11 +1325,15 @@ showToken t = case t of
   GENERATED         -> "{-# GENERATED"
   CORE              -> "{-# CORE"
   UNPACK            -> "{-# UNPACK"
-  OPTIONS (mt,s)    -> "{-# OPTIONS" ++ maybe "" (':':) mt ++ " ..."
+  OPTIONS (mt,_)    -> "{-# OPTIONS" ++ maybe "" (':':) mt ++ " ..."
 --  CFILES  s         -> "{-# CFILES ..."
 --  INCLUDE s         -> "{-# INCLUDE ..."
   LANGUAGE          -> "{-# LANGUAGE"
   ANN               -> "{-# ANN"
+  MINIMAL           -> "{-# MINIMAL"
+  NO_OVERLAP        -> "{-# NO_OVERLAP"
+  OVERLAP           -> "{-# OVERLAP"
+  INCOHERENT        -> "{-# INCOHERENT"
   KW_As         -> "as"
   KW_By         -> "by"
   KW_Case       -> "case"
@@ -1306,5 +1374,11 @@ showToken t = case t of
   KW_Interruptible -> "interruptible"
   KW_StdCall    -> "stdcall"
   KW_CCall      -> "ccall"
+  XChildTagOpen -> "<%>"
+  KW_CPlusPlus  -> "cplusplus"
+  KW_DotNet     -> "dotnet"
+  KW_Jvm        -> "jvm"
+  KW_Js         -> "js"
+  KW_CApi       -> "capi"
 
   EOF           -> "EOF"
