@@ -22,9 +22,10 @@ module Language.Haskell.Exts.ParseMonad(
         getSrcLoc, pushCurrentContext, popContext,
         getExtensions,
         -- * Lexing
-        Lex(runL), getInput, discard, lexNewline, lexTab, lexWhile,
+        Lex(runL), getInput, discard, lexNewline, lexTab, lexWhile, lexWhile_,
         alternative, checkBOL, setBOL, startToken, getOffside,
-        pushContextL, popContextL, getExtensionsL, pushComment, 
+        pushContextL, popContextL, getExtensionsL, addExtensionL,
+        saveExtensionsL, restoreExtensionsL, pushComment,
         getSrcLocL, setSrcLineL, ignoreLinePragmasL, setLineFilenameL,
         -- * Harp/Hsx
         ExtContext(..),
@@ -33,14 +34,14 @@ module Language.Haskell.Exts.ParseMonad(
         getModuleName
     ) where
 
-import Language.Haskell.Exts.SrcLoc(SrcLoc(..))
+import Language.Haskell.Exts.SrcLoc (SrcLoc(..), noLoc)
 import Language.Haskell.Exts.Fixity (Fixity, preludeFixities)
 import Language.Haskell.Exts.Comments
 import Language.Haskell.Exts.Extension -- (Extension, impliesExts, haskell2010)
 
-import Data.List ( intersperse )
+import Data.List (intercalate)
 import Control.Applicative
-import Control.Monad (when)
+import Control.Monad (when, liftM, ap)
 import Data.Monoid
 
 -- | The result of a parse.
@@ -60,7 +61,7 @@ fromParseResult (ParseFailed loc str) = error $ "fromParseResult: Parse failed a
 
 instance Functor ParseResult where
   fmap f (ParseOk x)           = ParseOk $ f x
-  fmap f (ParseFailed loc msg) = ParseFailed loc msg
+  fmap _ (ParseFailed loc msg) = ParseFailed loc msg
 
 instance Applicative ParseResult where
   pure = ParseOk
@@ -69,13 +70,14 @@ instance Applicative ParseResult where
 
 instance Monad ParseResult where
   return = ParseOk
+  fail = ParseFailed noLoc
   ParseOk x           >>= f = f x
   ParseFailed loc msg >>= _ = ParseFailed loc msg
 
 instance Monoid m => Monoid (ParseResult m) where
   mempty = ParseOk mempty
   ParseOk x `mappend` ParseOk y = ParseOk $ x `mappend` y
-  ParseOk x `mappend` err       = err
+  ParseOk _ `mappend` err       = err
   err       `mappend` _         = err -- left-biased
 
 
@@ -94,11 +96,11 @@ type CtxtFlag = (Bool,Bool)
 -- (True,_) = We're in a do context.
 -- (_, True)= Next token must be a virtual closing brace.
 
-type ParseState = ([LexContext],[ExtContext],CtxtFlag,[Comment])
+type ParseState = ([LexContext],[[KnownExtension]],[ExtContext],CtxtFlag,[Comment])
 
 indentOfParseState :: ParseState -> Int
-indentOfParseState (Layout n:_,_,_,_) = n
-indentOfParseState _                  = 0
+indentOfParseState (Layout n:_,_,_,_,_) = n
+indentOfParseState _                    = 0
 
 -- | Static parameters governing a parse.
 --   Note that the various parse functions in "Language.Haskell.Exts.Parser"
@@ -145,14 +147,14 @@ defaultParseMode = ParseMode {
 data InternalParseMode = IParseMode {
         iParseFilename :: String,
         iExtensions :: [KnownExtension],
-        iIgnoreLanguagePragmas :: Bool,
-        iIgnoreLinePragmas :: Bool,
-        iFixities :: Maybe [Fixity]
+        -- iIgnoreLanguagePragmas :: Bool,
+        iIgnoreLinePragmas :: Bool
+        -- iFixities :: Maybe [Fixity]
     }
 
 toInternalParseMode :: ParseMode -> InternalParseMode
-toInternalParseMode (ParseMode pf bLang exts ilang iline fx) =
-    IParseMode pf (impliesExts $ toExtensionList bLang exts) ilang iline fx
+toInternalParseMode (ParseMode pf bLang exts _ilang iline _fx) =
+    IParseMode pf (impliesExts $ toExtensionList bLang exts) {-_ilang -} iline {- _fx -}
 
 
 -- | Monad for parsing
@@ -184,9 +186,9 @@ runParser = runParserWithMode defaultParseMode
 
 runParserWithModeComments :: ParseMode -> P a -> String -> ParseResult (a, [Comment])
 runParserWithModeComments mode (P m) s = 
-  case m s 0 1 start ([],[],(False,False),[]) (toInternalParseMode mode) of
-    Ok (_,_,_,cs) a -> ParseOk (a, reverse cs)
-    Failed loc msg -> ParseFailed loc msg
+  case m s 0 1 start ([],[],[],(False,False),[]) (toInternalParseMode mode) of
+    Ok (_,_,_,_,cs) a -> ParseOk (a, reverse cs)
+    Failed loc msg    -> ParseFailed loc msg
     where start = SrcLoc {
         srcFilename = parseFilename mode,
         srcLine = 1,
@@ -196,6 +198,13 @@ runParserWithModeComments mode (P m) s =
 
     --      allExts mode = let imode = to
 
+instance Functor P where
+  fmap = liftM
+
+instance Applicative P where
+  pure = return
+  (<*>) = ap
+
 instance Monad P where
     return a = P $ \_i _x _y _l s _m -> Ok s a
     P m >>= k = P $ \i x y l s mode ->
@@ -213,7 +222,7 @@ getSrcLoc = P $ \_i _x _y l s _m -> Ok s l
 getModuleName :: P String
 getModuleName = P $ \_i _x _y _l s m ->
     let fn = iParseFilename m
-        mn = concat $ intersperse "." $ splitPath fn
+        mn = intercalate "." $ splitPath fn
 
         splitPath :: String -> [String]
         splitPath ""   = []
@@ -238,25 +247,25 @@ pushCurrentContext = do
     dob <- pullDoStatus
     let loc = srcColumn lc
     when (dob && loc < indent
-           || not dob && loc <= indent) $ pushCtxtFlag
+           || not dob && loc <= indent) pushCtxtFlag
     pushContext (Layout loc)
 
 currentIndent :: P Int
-currentIndent = P $ \_r _x _y loc stk _mode -> Ok stk (indentOfParseState stk)
+currentIndent = P $ \_r _x _y _ stk _mode -> Ok stk (indentOfParseState stk)
 
 pushContext :: LexContext -> P ()
 pushContext ctxt =
 --trace ("pushing lexical scope: " ++ show ctxt ++"\n") $
-    P $ \_i _x _y _l (s, e, p, c) _m -> Ok (ctxt:s, e, p, c) ()
+    P $ \_i _x _y _l (s, exts, e, p, c) _m -> Ok (ctxt:s, exts, e, p, c) ()
 
 popContext :: P ()
 popContext = P $ \_i _x _y loc stk _m ->
       case stk of
-        (_:s, e, p, c) -> --trace ("popping lexical scope, context now "++show s ++ "\n") $
-                          Ok (s, e, p, c) ()
-        ([],_,_,_)     -> Failed loc "Unexpected }" -- error "Internal error: empty context in popContext"
-
+        (_:s, exts, e, p, c) -> --trace ("popping lexical scope, context now "++show s ++ "\n") $
+                          Ok (s, exts, e, p, c) ()
+        ([],_,_,_,_)   -> Failed loc "Unexpected }" -- error "Internal error: empty context in popContext"
 
+{-
 -- HaRP/Hsx
 pushExtContext :: ExtContext -> P ()
 pushExtContext ctxt = P $ \_i _x _y _l (s, e, p, c) _m -> Ok (s, ctxt:e, p, c) ()
@@ -267,7 +276,7 @@ popExtContext = P $ \_i _x _y _l (s, e, p, c) _m ->
      (_:e') ->
        Ok (s, e', p, c) ()
      [] -> error "Internal error: empty context in popExtContext"
-
+-}
 
 -- Extension-aware lexing/parsing
 getExtensions :: P [KnownExtension]
@@ -276,12 +285,12 @@ getExtensions = P $ \_i _x _y _l s m ->
 
 pushCtxtFlag :: P ()
 pushCtxtFlag =
-    P $ \_i _x _y _l (s, e, (d,c), cs) _m -> case c of
-        False -> Ok (s, e, (d,True), cs) ()
+    P $ \_i _x _y _l (s, exts, e, (d,c), cs) _m -> case c of
+        False -> Ok (s, exts, e, (d,True), cs) ()
         _     -> error "Internal error: context flag already pushed"
 
 pullDoStatus :: P Bool
-pullDoStatus = P $ \_i _x _y _l (s, e, (d,c), cs) _m -> Ok (s,e,(False,c),cs) d
+pullDoStatus = P $ \_i _x _y _l (s, exts, e, (d,c), cs) _m -> Ok (s,exts,e,(False,c),cs) d
 
 
 ----------------------------------------------------------------------------
@@ -290,6 +299,13 @@ pullDoStatus = P $ \_i _x _y _l (s, e, (d,c), cs) _m -> Ok (s,e,(False,c),cs) d
 
 newtype Lex r a = Lex { runL :: (a -> P r) -> P r }
 
+instance Functor (Lex r) where
+    fmap = liftM
+
+instance Applicative (Lex r) where
+    pure = return
+    (<*>) = ap
+
 instance Monad (Lex r) where
     return a = Lex $ \k -> k a
     Lex v >>= f = Lex $ \k -> v (\a -> runL (f a) k)
@@ -323,7 +339,7 @@ nextTab :: Int -> Int
 nextTab x = x + (tAB_LENGTH - (x-1) `mod` tAB_LENGTH)
 
 tAB_LENGTH :: Int
-tAB_LENGTH = 8 :: Int
+tAB_LENGTH = 8
 
 -- Consume and return the largest string of characters satisfying p
 
@@ -332,6 +348,11 @@ lexWhile p = Lex $ \cont -> P $ \r x ->
     let (cs,rest) = span p r in
     runP (cont cs) rest (x + length cs)
 
+-- | lexWhile without the return value.
+lexWhile_ :: (Char -> Bool) -> Lex a ()
+lexWhile_ p = do _ <- lexWhile p
+                 return ()
+
 -- An alternative scan, to which we can return if subsequent scanning
 -- is unsuccessful.
 
@@ -392,40 +413,40 @@ setSrcLineL y = Lex $ \cont -> P $ \i x _ ->
         runP (cont ()) i x y
 
 pushContextL :: LexContext -> Lex a ()
-pushContextL ctxt = Lex $ \cont -> P $ \r x y loc (stk, e, pst, cs) ->
-        runP (cont ()) r x y loc (ctxt:stk, e, pst, cs)
+pushContextL ctxt = Lex $ \cont -> P $ \r x y loc (stk, exts, e, pst, cs) ->
+        runP (cont ()) r x y loc (ctxt:stk, exts, e, pst, cs)
 
 popContextL :: String -> Lex a ()
-popContextL fn = Lex $ \cont -> P $ \r x y loc stk m -> case stk of
-        (_:ctxt, e, pst, cs) -> runP (cont ()) r x y loc (ctxt, e, pst, cs) m
-        ([], _, _, _)        -> Failed loc "Unexpected }"
+popContextL _ = Lex $ \cont -> P $ \r x y loc stk m -> case stk of
+        (_:ctxt, exts, e, pst, cs) -> runP (cont ()) r x y loc (ctxt, exts, e, pst, cs) m
+        ([], _, _, _, _)           -> Failed loc "Unexpected }"
 
 pullCtxtFlag :: Lex a Bool
-pullCtxtFlag = Lex $ \cont -> P $ \r x y loc (ct, e, (d,c), cs) ->
-        runP (cont c) r x y loc (ct, e, (d,False), cs)
+pullCtxtFlag = Lex $ \cont -> P $ \r x y loc (ct, exts, e, (d,c), cs) ->
+        runP (cont c) r x y loc (ct, exts, e, (d,False), cs)
 
 
 flagDo :: Lex a ()
-flagDo = Lex $ \cont -> P $ \r x y loc (ct, e, (d,c), cs) ->
-        runP (cont ()) r x y loc (ct, e, (True,c), cs)
+flagDo = Lex $ \cont -> P $ \r x y loc (ct, exts, e, (_,c), cs) ->
+        runP (cont ()) r x y loc (ct, exts, e, (True,c), cs)
 
 
 -- Harp/Hsx
 
 getExtContext :: Lex a (Maybe ExtContext)
-getExtContext = Lex $ \cont -> P $ \r x y loc stk@(_, e, _, _) ->
+getExtContext = Lex $ \cont -> P $ \r x y loc stk@(_, _, e, _, _) ->
         let me = case e of
               [] -> Nothing
               (c:_) -> Just c
         in runP (cont me) r x y loc stk
 
 pushExtContextL :: ExtContext -> Lex a ()
-pushExtContextL ec = Lex $ \cont -> P $ \r x y loc (s, e, p, c) ->
-        runP (cont ()) r x y loc (s, ec:e, p, c)
+pushExtContextL ec = Lex $ \cont -> P $ \r x y loc (s, exts, e, p, c) ->
+        runP (cont ()) r x y loc (s, exts, ec:e, p, c)
 
 popExtContextL :: String -> Lex a ()
-popExtContextL fn = Lex $ \cont -> P $ \r x y loc stk@(s,e,p,c) m -> case e of
-            (_:ec) -> runP (cont ()) r x y loc (s,ec,p,c) m
+popExtContextL fn = Lex $ \cont -> P $ \r x y loc (s,exts,e,p,c) m -> case e of
+            (_:ec)   -> runP (cont ()) r x y loc (s,exts,ec,p,c) m
             []       -> Failed loc ("Internal error: empty tag context in " ++ fn)
 
 
@@ -435,6 +456,23 @@ getExtensionsL :: Lex a [KnownExtension]
 getExtensionsL = Lex $ \cont -> P $ \r x y loc s m ->
         runP (cont $ iExtensions m) r x y loc s m
 
+-- | Add an extension to the current configuration.
+addExtensionL :: KnownExtension -> Lex a ()
+addExtensionL ext = Lex $ \cont -> P $ \r x y loc (s, oldExts, e, p, c) m ->
+        let newExts = impliesExts [ext] ++ iExtensions m
+        in runP (cont ()) r x y loc (s, oldExts, e, p, c) (m {iExtensions = newExts})
+
+-- | Save the current configuration of extensions.
+saveExtensionsL :: Lex a ()
+saveExtensionsL = Lex $ \cont -> P $ \r x y loc (s, oldExts, e, p, c) m ->
+        runP (cont ()) r x y loc (s, iExtensions m:oldExts, e, p, c) m
+
+-- | Return to the previous saved extensions configuration.
+restoreExtensionsL :: Lex a ()
+restoreExtensionsL = Lex $ \cont -> P $ \r x y loc (s,exts,e,p,c) m -> case exts of
+            (_:prev) -> runP (cont ()) r x y loc (s,prev,e,p,c) m
+            _        -> Failed loc "Internal error: empty extension stack"
+
 -- LINE-aware lexing
 
 ignoreLinePragmasL :: Lex a Bool
@@ -449,5 +487,5 @@ setLineFilenameL name = Lex $ \cont -> P $ \r x y loc s m ->
 -- Comments
 
 pushComment :: Comment -> Lex a ()
-pushComment c = Lex $ \cont -> P $ \r x y loc (s, e, p, cs) ->
-        runP (cont ()) r x y loc (s, e, p, c:cs)
+pushComment c = Lex $ \cont -> P $ \r x y loc (s, exts, e, p, cs) ->
+        runP (cont ()) r x y loc (s, exts, e, p, c:cs)
