@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, DeriveDataTypeable #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.Exts.Syntax
@@ -40,7 +40,7 @@
 module Language.Haskell.Exts.Syntax (
     -- * Modules
     Module(..), WarningText(..), ExportSpec(..),
-    ImportDecl(..), ImportSpec(..), Assoc(..),
+    ImportDecl(..), ImportSpec(..), Assoc(..), Namespace(..),
     -- * Declarations
     Decl(..), Binds(..), IPBind(..),
     -- ** Type classes and instances
@@ -52,14 +52,15 @@ module Language.Haskell.Exts.Syntax (
     -- * Class Assertions and Contexts
     Context, FunDep(..), Asst(..),
     -- * Types
-    Type(..), Boxed(..), Kind(..), TyVarBind(..),
+    Type(..), Boxed(..), Kind(..), TyVarBind(..), Promoted(..),
+    TypeEqn (..),
     -- * Expressions
     Exp(..), Stmt(..), QualStmt(..), FieldUpdate(..),
-    Alt(..), GuardedAlts(..), GuardedAlt(..), XAttr(..),
+    Alt(..), XAttr(..),
     -- * Patterns
     Pat(..), PatField(..), PXAttr(..), RPat(..), RPatOp(..),
     -- * Literals
-    Literal(..),
+    Literal(..), Sign(..),
     -- * Variables, Constructors and Operators
     ModuleName(..), QName(..), Name(..), QOp(..), Op(..),
     SpecialCon(..), CName(..), IPName(..), XName(..),
@@ -71,9 +72,9 @@ module Language.Haskell.Exts.Syntax (
     Safety(..), CallConv(..),
 
     -- * Pragmas
-    ModulePragma(..), Tool(..),
+    ModulePragma(..), Tool(..), Overlap(..),
     Rule(..), RuleVar(..), Activation(..),
-    Annotation(..),
+    Annotation(..), BooleanFormula(..),
 
     -- * Builtin names
 
@@ -86,9 +87,9 @@ module Language.Haskell.Exts.Syntax (
     unit_con, tuple_con, unboxed_singleton_con,
     -- ** Special identifiers
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
-    export_name, safe_name, unsafe_name, threadsafe_name,
+    export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
     stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
-    forall_name, family_name,
+    capi_name, forall_name, family_name,
     -- ** Type constructors
     unit_tycon_name, fun_tycon_name, list_tycon_name, tuple_tycon_name, unboxed_singleton_tycon_name,
     unit_tycon, fun_tycon, list_tycon, tuple_tycon, unboxed_singleton_tycon,
@@ -98,26 +99,15 @@ module Language.Haskell.Exts.Syntax (
   ) where
 
 
-#ifdef __GLASGOW_HASKELL__
-#ifdef BASE4
 import Data.Data
-#else
-import Data.Generics (Data(..),Typeable(..))
-#endif
-#endif
-
+import GHC.Generics (Generic)
 import Language.Haskell.Exts.SrcLoc (SrcLoc(..))
-
 import Language.Haskell.Exts.Annotated.Syntax (Boxed(..), Tool(..))
 
 
 -- | The name of a Haskell module.
 newtype ModuleName = ModuleName String
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Constructors with special syntax.
 -- These names are never qualified, and always refer to builtin type or
@@ -130,11 +120,7 @@ data SpecialCon
                             --   constructors @(,)@ etc, possibly boxed @(\#,\#)@
     | Cons                  -- ^ list data constructor @(:)@
     | UnboxedSingleCon      -- ^ unboxed singleton tuple constructor @(\# \#)@
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | This type is used to represent qualified variables, and also
 --   qualified constructors.
@@ -142,75 +128,47 @@ data QName
     = Qual ModuleName Name    -- ^ name qualified with a module name
     | UnQual Name             -- ^ unqualified local name
     | Special SpecialCon      -- ^ built-in constructor with special syntax
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | This type is used to represent variables, and also constructors.
 data Name
     = Ident String    -- ^ /varid/ or /conid/.
     | Symbol String   -- ^ /varsym/ or /consym/
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An implicit parameter name.
 data IPName
     = IPDup String -- ^ ?/ident/, non-linear implicit parameter
     | IPLin String -- ^ %/ident/, linear implicit parameter
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Possibly qualified infix operators (/qop/), appearing in expressions.
 data QOp
     = QVarOp QName  -- ^ variable operator (/qvarop/)
     | QConOp QName  -- ^ constructor operator (/qconop/)
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Operators appearing in @infix@ declarations are never qualified.
 data Op
     = VarOp Name    -- ^ variable operator (/varop/)
     | ConOp Name    -- ^ constructor operator (/conop/)
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A name (/cname/) of a component of a class or data type in an @import@
 --   or export specification.
 data CName
     = VarName Name  -- ^ name of a method or field
     | ConName Name  -- ^ name of a data constructor
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A complete Haskell source module.
 data Module = Module SrcLoc ModuleName [ModulePragma] (Maybe WarningText)
                         (Maybe [ExportSpec]) [ImportDecl] [Decl]
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An item in a module's export specification.
 data ExportSpec
-     = EVar QName                   -- ^ variable
+     = EVar Namespace QName         -- ^ variable.
      | EAbs QName                   -- ^ @T@:
                                     --   a class or datatype exported abstractly,
                                     --   or a type synonym.
@@ -222,11 +180,11 @@ data ExportSpec
                                     --   a datatype exported with some of its constructors.
      | EModuleContents ModuleName   -- ^ @module M@:
                                     --   re-export a module.
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+-- | Namespaces for imports/exports.
+data Namespace = NoNamespace | TypeNamespace
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An import declaration.
 data ImportDecl = ImportDecl
@@ -234,6 +192,7 @@ data ImportDecl = ImportDecl
     , importModule :: ModuleName    -- ^ name of the module imported.
     , importQualified :: Bool       -- ^ imported @qualified@?
     , importSrc :: Bool             -- ^ imported with @{-\# SOURCE \#-}@?
+    , importSafe :: Bool            -- ^ Import @safe@?
     , importPkg :: Maybe String     -- ^ imported with explicit package name
     , importAs :: Maybe ModuleName  -- ^ optional alias name in an @as@ clause.
     , importSpecs :: Maybe (Bool,[ImportSpec])
@@ -241,16 +200,12 @@ data ImportDecl = ImportDecl
             -- The 'Bool' is 'True' if the names are excluded
             -- by @hiding@.
     }
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An import specification, representing a single explicit item imported
 --   (or hidden) from a module.
 data ImportSpec
-     = IVar Name                -- ^ variable
+     = IVar Namespace Name      -- ^ variable.
      | IAbs Name                -- ^ @T@:
                                 --   the name of a class, datatype or type synonym.
      | IThingAll Name           -- ^ @T(..)@:
@@ -259,22 +214,14 @@ data ImportSpec
      | IThingWith Name [CName]  -- ^ @T(C_1,...,C_n)@:
                                 --   a class imported with some of its methods, or
                                 --   a datatype imported with some of its constructors.
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Associativity of an operator.
 data Assoc
      = AssocNone  -- ^ non-associative operator (declared with @infix@)
      | AssocLeft  -- ^ left-associative operator (declared with @infixl@).
      | AssocRight -- ^ right-associative operator (declared with @infixr@)
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A single derived instance, which may have arguments since it may be a MPTC.
 type Deriving = (QName, [Type])
@@ -285,6 +232,8 @@ data Decl
      -- ^ A type declaration
      | TypeFamDecl  SrcLoc Name [TyVarBind] (Maybe Kind)
      -- ^ A type family declaration
+     | ClosedTypeFamDecl  SrcLoc Name [TyVarBind] (Maybe Kind) [TypeEqn]
+     -- ^ A closed type family declaration
      | DataDecl     SrcLoc DataOrNew Context Name [TyVarBind]              [QualConDecl] [Deriving]
      -- ^ A data OR newtype declaration
      | GDataDecl    SrcLoc DataOrNew Context Name [TyVarBind] (Maybe Kind) [GadtDecl]    [Deriving]
@@ -299,9 +248,9 @@ data Decl
      -- ^ A data family instance declaration, GADT style
      | ClassDecl    SrcLoc Context Name [TyVarBind] [FunDep] [ClassDecl]
      -- ^ A declaration of a type class
-     | InstDecl     SrcLoc Context QName [Type] [InstDecl]
+     | InstDecl     SrcLoc (Maybe Overlap) [TyVarBind] Context QName [Type] [InstDecl]
      -- ^ An declaration of a type class instance
-     | DerivDecl    SrcLoc Context QName [Type]
+     | DerivDecl    SrcLoc (Maybe Overlap) [TyVarBind] Context QName [Type]
      -- ^ A standalone deriving declaration
      | InfixDecl    SrcLoc Assoc Int [Op]
      -- ^ A declaration of operator fixity
@@ -313,7 +262,7 @@ data Decl
      -- ^ A type signature declaration
      | FunBind      [Match]
      -- ^ A set of function binding clauses
-     | PatBind      SrcLoc Pat (Maybe Type) Rhs {-where-} Binds
+     | PatBind      SrcLoc Pat Rhs {-where-} Binds
      -- ^ A pattern binding
      | ForImp   SrcLoc CallConv Safety String Name Type
      -- ^ A foreign import declaration
@@ -334,15 +283,16 @@ data Decl
      -- ^ A SPECIALISE pragma
      | SpecInlineSig    SrcLoc Bool Activation QName [Type]
      -- ^ A SPECIALISE INLINE pragma
-     | InstSig          SrcLoc Context         QName [Type]
+     | InstSig          SrcLoc [TyVarBind] Context QName [Type]
      -- ^ A SPECIALISE instance pragma
      | AnnPragma        SrcLoc Annotation
      -- ^ An ANN pragma
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+     | MinimalPragma    SrcLoc (Maybe BooleanFormula)
+     -- ^ A MINIMAL pragma
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+-- | A type equation of the form @rhs = lhs@ used in closed type families.
+data TypeEqn = TypeEqn Type Type deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An annotation through an ANN pragma.
 data Annotation
@@ -352,46 +302,34 @@ data Annotation
     -- ^ An annotation for a declared type.
     | ModuleAnn      Exp
     -- ^ An annotation for the defining module.
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+-- | A boolean formula for MINIMAL pragmas.
+data BooleanFormula
+    = VarFormula Name                -- ^ A variable.
+    | AndFormula [BooleanFormula]    -- ^ And boolean formulas.
+    | OrFormula [BooleanFormula]     -- ^ Or boolean formulas.
+    | ParenFormula BooleanFormula    -- ^ Parenthesized boolean formulas.
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A flag stating whether a declaration is a data or newtype declaration.
 data DataOrNew = DataType | NewType
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A binding group inside a @let@ or @where@ clause.
 data Binds
     = BDecls [Decl]     -- ^ An ordinary binding group
     | IPBinds [IPBind]  -- ^ A binding group for implicit parameters
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A binding of an implicit parameter.
 data IPBind = IPBind SrcLoc IPName Exp
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Clauses of a function binding.
 data Match
      = Match SrcLoc Name [Pat] (Maybe Type) Rhs {-where-} Binds
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A single constructor declaration within a data type declaration,
 --   which may have an existential quantification binding.
@@ -399,34 +337,22 @@ data QualConDecl
     = QualConDecl SrcLoc
         {-forall-} [TyVarBind] {- . -} Context
         {- => -} ConDecl
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Declaration of an ordinary data constructor.
 data ConDecl
-     = ConDecl Name [BangType]
+     = ConDecl Name [Type]
                 -- ^ ordinary data constructor
-     | InfixConDecl BangType Name BangType
+     | InfixConDecl Type Name Type
                 -- ^ infix data constructor
-     | RecDecl Name [([Name],BangType)]
+     | RecDecl Name [([Name],Type)]
                 -- ^ record constructor
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A single constructor declaration in a GADT data type declaration.
 data GadtDecl
-    = GadtDecl SrcLoc Name Type
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+    = GadtDecl SrcLoc Name [([Name], Type)] Type
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Declarations inside a class declaration.
 data ClassDecl
@@ -438,11 +364,9 @@ data ClassDecl
             -- ^ declaration of an associated type synonym
     | ClsTyDef   SrcLoc Type    Type
             -- ^ default choice for an associated type synonym
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+    | ClsDefSig  SrcLoc Name Type
+            -- ^ default signature
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Declarations inside an instance declaration.
 data InstDecl
@@ -454,45 +378,30 @@ data InstDecl
             -- ^ an associated data type implementation
     | InsGData  SrcLoc DataOrNew Type (Maybe Kind) [GadtDecl] [Deriving]
             -- ^ an associated data type implemented using GADT style
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The type of a constructor argument or field, optionally including
 --   a strictness annotation.
 data BangType
-     = BangedTy   Type  -- ^ strict component, marked with \"@!@\"
-     | UnBangedTy Type  -- ^ non-strict component
-     | UnpackedTy Type  -- ^ unboxed component, marked with an UNPACK pragma
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+     = BangedTy    -- ^ strict component, marked with \"@!@\"
+     | UnpackedTy  -- ^ unboxed component, marked with an UNPACK pragma
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
--- | The right hand side of a function or pattern binding.
+-- | The right hand side of a function binding, pattern binding, or a case
+--   alternative.
 data Rhs
      = UnGuardedRhs Exp -- ^ unguarded right hand side (/exp/)
      | GuardedRhss  [GuardedRhs]
                         -- ^ guarded right hand side (/gdrhs/)
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
--- | A guarded right hand side @|@ /stmts/ @=@ /exp/.
+-- | A guarded right hand side @|@ /stmts/ @=@ /exp/, or @|@ /stmts/ @->@ /exp/
+--   for case alternatives.
 --   The guard is a series of statements when using pattern guards,
 --   otherwise it will be a single qualifier expression.
 data GuardedRhs
      = GuardedRhs SrcLoc [Stmt] Exp
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A type qualified with a context.
 --   An unqualified type has an empty context.
@@ -504,27 +413,33 @@ data Type
      | TyFun   Type Type        -- ^ function type
      | TyTuple Boxed [Type]     -- ^ tuple type, possibly boxed
      | TyList  Type             -- ^ list syntax, e.g. [a], as opposed to [] a
+     | TyParArray Type          -- ^ parallel array syntax, e.g. [:a:]
      | TyApp   Type Type        -- ^ application of a type constructor
      | TyVar   Name             -- ^ type variable
      | TyCon   QName            -- ^ named type or type constructor
      | TyParen Type             -- ^ type surrounded by parentheses
      | TyInfix Type QName Type  -- ^ infix type constructor
      | TyKind  Type Kind        -- ^ type with explicit kind signature
-#ifdef __GLASGOW_HASKELL__
+     | TyPromoted Promoted      -- ^ promoted data type (-XDataKinds)
+     | TyEquals Type Type       -- ^ type equality predicate enabled by ConstraintKinds
+     | TySplice Splice          -- ^ template haskell splice type
+     | TyBang BangType Type     -- ^ Strict type marked with \"@!@\" or type marked with UNPACK pragma.
   deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+
+data Promoted
+        = PromotedInteger Integer
+        | PromotedString String
+        | PromotedCon Bool QName
+        | PromotedList Bool [Promoted]
+        | PromotedTuple [Promoted]
+        | PromotedUnit
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A type variable declaration, optionally with an explicit kind annotation.
 data TyVarBind
     = KindedVar Name Kind   -- ^ variable binding with kind annotation
     | UnkindedVar Name      -- ^ ordinary variable binding
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An explicit kind annotation.
 data Kind
@@ -532,23 +447,18 @@ data Kind
     | KindBang          -- ^ @!@, the kind of unboxed types
     | KindFn Kind Kind  -- ^ @->@, the kind of a type constructor
     | KindParen Kind    -- ^ a kind surrounded by parentheses
-    | KindVar Name      -- ^ a kind variable (as of yet unsupported by compilers)
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+    | KindVar QName     -- ^ a kind variable (as of yet unsupported by compilers)
+    | KindApp Kind Kind -- ^ @k1 k2@
+    | KindTuple [Kind]  -- ^ @'(k1,k2,k3)@, a promoted tuple
+    | KindList  [Kind]  -- ^ @'[k1,k2,k3]@, a promoted list literal
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 
 -- | A functional dependency, given on the form
 --   l1 l2 ... ln -> r2 r3 .. rn
 data FunDep
     = FunDep [Name] [Name]
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A context is a set of assertions
 type Context = [Asst]
@@ -558,14 +468,12 @@ type Context = [Asst]
 --   allows multiple parameters, and allows them to be /type/s.
 --   Also extended with support for implicit parameters and equality constraints.
 data Asst = ClassA QName [Type]     -- ^ ordinary class assertion
+          | VarA Name               -- ^ constraint kind assertion, @Dict :: cxt => Dict cxt@
           | InfixA Type QName Type  -- ^ class assertion where the class name is given infix
           | IParam IPName Type      -- ^ implicit parameter assertion
           | EqualP Type   Type      -- ^ type equality constraint
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+          | ParenA Asst             -- ^ parenthesised class assertion
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | /literal/
 -- Values of this type hold the abstract value of the literal, not the
@@ -582,11 +490,13 @@ data Literal
     | PrimDouble Rational   -- ^ unboxed double literal
     | PrimChar   Char       -- ^ unboxed character literal
     | PrimString String     -- ^ unboxed string literal
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+-- | An indication whether a literal pattern has been negated or not.
+data Sign
+    = Signless
+    | Negative
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Haskell expressions.
 data Exp
@@ -600,6 +510,7 @@ data Exp
     | Lambda SrcLoc [Pat] Exp   -- ^ lambda expression
     | Let Binds Exp             -- ^ local declarations with @let@ ... @in@ ...
     | If Exp Exp Exp            -- ^ @if@ /exp/ @then@ /exp/ @else@ /exp/
+    | MultiIf [GuardedRhs]      -- ^ @if@ @|@ /exp/ @->@ /exp/ ...
     | Case Exp [Alt]            -- ^ @case@ /exp/ @of@ /alts/
     | Do [Stmt]                 -- ^ @do@-expression:
                                 --   the last statement in the list
@@ -608,6 +519,7 @@ data Exp
     | Tuple Boxed [Exp]         -- ^ tuple expression
     | TupleSection Boxed [Maybe Exp]  -- ^ tuple section expression, e.g. @(,,3)@
     | List [Exp]                -- ^ list expression
+    | ParArray [Exp]            -- ^ parallel array expression
     | Paren Exp                 -- ^ parenthesised expression
     | LeftSection Exp QOp       -- ^ left section @(@/exp/ /qop/@)@
     | RightSection QOp Exp      -- ^ right section @(@/qop/ /exp/@)@
@@ -624,8 +536,15 @@ data Exp
     | EnumFromThenTo Exp Exp Exp
                                 -- ^ bounded arithmetic sequence,
                                 --   with first two elements given @[from, then .. to]@
+    | ParArrayFromTo Exp Exp    -- ^ bounded arithmetic sequence,
+                                --   incrementing by 1 @[from .. to]@
+    | ParArrayFromThenTo Exp Exp Exp
+                                -- ^ bounded arithmetic sequence,
+                                --   with first two elements given @[from, then .. to]@
     | ListComp Exp  [QualStmt]    -- ^ ordinary list comprehension
     | ParComp  Exp [[QualStmt]]   -- ^ parallel list comprehension
+    | ParArrayComp  Exp [[QualStmt]]
+                                  -- ^ parallel array comprehension
     | ExpTypeSig SrcLoc Exp Type  -- ^ expression with explicit type signature
 
     | VarQuote QName            -- ^ @'x@ for template haskell reifying of expressions
@@ -655,30 +574,21 @@ data Exp
     | RightArrApp     Exp Exp   -- ^ arrow application (from right): /exp/ @>-@ /exp/
     | LeftArrHighApp  Exp Exp   -- ^ higher-order arrow application (from left): /exp/ @-<<@ /exp/
     | RightArrHighApp Exp Exp   -- ^ higher-order arrow application (from right): /exp/ @>>-@ /exp/
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+
+-- LambdaCase
+    | LCase [Alt] -- ^ @\case@ /alts/
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The name of an xml element or attribute,
 --   possibly qualified with a namespace.
 data XName
     = XName String              -- <name ...
     | XDomName String String    -- <dom:name ...
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An xml attribute, which is a name-expression pair.
 data XAttr = XAttr XName Exp
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A template haskell bracket expression.
 data Bracket
@@ -686,32 +596,20 @@ data Bracket
     | PatBracket Pat        -- ^ pattern bracket: @[p| ... |]@
     | TypeBracket Type      -- ^ type bracket: @[t| ... |]@
     | DeclBracket [Decl]    -- ^ declaration bracket: @[d| ... |]@
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A template haskell splice expression
 data Splice
     = IdSplice String       -- ^ variable splice: @$var@
     | ParenSplice Exp       -- ^ parenthesised expression splice: @$(/exp/)@
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The safety of a foreign function call.
 data Safety
     = PlayRisky         -- ^ unsafe
     | PlaySafe Bool     -- ^ safe ('False') or threadsafe ('True')
     | PlayInterruptible -- ^ interruptible
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The calling convention of a foreign function call.
 data CallConv
@@ -722,11 +620,7 @@ data CallConv
     | Jvm
     | Js
     | CApi
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A top level options pragma, preceding the module header.
 data ModulePragma
@@ -735,59 +629,45 @@ data ModulePragma
                         -- ^ OPTIONS pragma, possibly qualified with a tool, e.g. OPTIONS_GHC
     | AnnModulePragma  SrcLoc Annotation
                         -- ^ ANN pragma with module scope
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
+
+-- | Recognised overlaps for overlap pragmas.
+data Overlap
+    = NoOverlap   -- ^ NO_OVERLAP pragma
+    | Overlap     -- ^ OVERLAP pragma
+    | Incoherent  -- ^ INCOHERENT pragma
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Activation clause of a RULES pragma.
 data Activation
     = AlwaysActive
     | ActiveFrom  Int
     | ActiveUntil Int
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | The body of a RULES pragma.
 data Rule
     = Rule String Activation (Maybe [RuleVar]) Exp Exp
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Variables used in a RULES pragma, optionally annotated with types
 data RuleVar
     = RuleVar Name
     | TypedRuleVar Name Type
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | Warning text to optionally use in the module header of e.g.
 --   a deprecated module.
 data WarningText
     = DeprText String
     | WarnText String
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 
 -- | A pattern, to be matched against a value.
 data Pat
     = PVar Name                     -- ^ variable
-    | PLit Literal                  -- ^ literal constant
-    | PNeg Pat                      -- ^ negated pattern
+    | PLit Sign Literal             -- ^ literal constant
     | PNPlusK Name Integer          -- ^ n+k pattern
     | PInfixApp Pat QName Pat       -- ^ pattern with an infix data constructor
     | PApp QName [Pat]              -- ^ data constructor and argument patterns
@@ -808,22 +688,13 @@ data Pat
     | PXPcdata String               -- ^ XML PCDATA pattern
     | PXPatTag Pat                  -- ^ XML embedded pattern
     | PXRPats [RPat]                -- ^ XML regular list pattern
-    | PExplTypeArg QName Type       -- ^ Explicit generics style type argument e.g. @f {| Int |} x = ...@
     | PQuasiQuote String String     -- ^ quasi quote patter: @[$/name/| /string/ |]@
     | PBangPat Pat                  -- ^ strict (bang) pattern: @f !x = ...@
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An XML attribute in a pattern.
 data PXAttr = PXAttr XName Pat
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A regular pattern operator.
 data RPatOp
@@ -833,11 +704,7 @@ data RPatOp
     | RPPlusG   -- ^ @+!@ = 1 or more, greedy
     | RPOpt     -- ^ @?@ = 0 or 1
     | RPOptG    -- ^ @?!@ = 0 or 1, greedy
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An entity in a regular pattern.
 data RPat
@@ -849,22 +716,14 @@ data RPat
     | RPAs Name RPat        -- ^ linear variable binding, e.g. foo\@(1 | 2)
     | RPParen RPat          -- ^ parenthesised pattern, e.g. (2*)
     | RPPat Pat             -- ^ an ordinary pattern
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An /fpat/ in a labeled record pattern.
 data PatField
     = PFieldPat QName Pat       -- ^ ordinary label-pattern pair
-    | PFieldPun Name            -- ^ record field pun
+    | PFieldPun QName           -- ^ record field pun
     | PFieldWildcard            -- ^ record field wildcard
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A statement, representing both a /stmt/ in a @do@-expression,
 --   an ordinary /qual/ in a list comprehension, as well as a /stmt/
@@ -878,11 +737,7 @@ data Stmt
                         --   a guard expression
     | LetStmt Binds     -- ^ local bindings
     | RecStmt [Stmt]    -- ^ a recursive binding group for arrows
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | A general /transqual/ in a list comprehension,
 --   which could potentially be a transform of the kind
@@ -894,52 +749,19 @@ data QualStmt
     | GroupBy      Exp          -- ^ @then@ @group@ @by@ /exp/
     | GroupUsing   Exp          -- ^ @then@ @group@ @using@ /exp/
     | GroupByUsing Exp Exp      -- ^ @then@ @group@ @by@ /exp/ @using@ /exp/
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An /fbind/ in a labeled construction or update expression.
 data FieldUpdate
     = FieldUpdate QName Exp     -- ^ ordinary label-expresion pair
-    | FieldPun Name             -- ^ record field pun
+    | FieldPun QName            -- ^ record field pun
     | FieldWildcard             -- ^ record field wildcard
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -- | An /alt/ alternative in a @case@ expression.
 data Alt
-    = Alt SrcLoc Pat GuardedAlts Binds
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
-
--- | The right-hand sides of a @case@ alternative,
---   which may be a single right-hand side or a
---   set of guarded ones.
-data GuardedAlts
-    = UnGuardedAlt Exp          -- ^ @->@ /exp/
-    | GuardedAlts  [GuardedAlt] -- ^ /gdpat/
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
-
--- | A guarded case alternative @|@ /stmts/ @->@ /exp/.
-data GuardedAlt
-    = GuardedAlt SrcLoc [Stmt] Exp
-#ifdef __GLASGOW_HASKELL__
-  deriving (Eq,Ord,Show,Typeable,Data)
-#else
-  deriving (Eq,Ord,Show)
-#endif
+    = Alt SrcLoc Pat Rhs Binds
+  deriving (Eq,Ord,Show,Typeable,Data,Generic)
 
 -----------------------------------------------------------------------------
 -- Builtin names.
@@ -970,7 +792,7 @@ tuple_con :: Boxed -> Int -> Exp
 tuple_con b i = Con (tuple_con_name b i)
 
 unboxed_singleton_con :: Exp
-unboxed_singleton_con = Con (unboxed_singleton_con_name)
+unboxed_singleton_con = Con unboxed_singleton_con_name
 
 as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name :: Name
 as_name        = Ident "as"
@@ -981,12 +803,13 @@ bang_name      = Symbol "!"
 dot_name       = Symbol "."
 star_name      = Symbol "*"
 
-export_name, safe_name, unsafe_name, threadsafe_name,
+export_name, safe_name, unsafe_name, interruptible_name, threadsafe_name,
   stdcall_name, ccall_name, cplusplus_name, dotnet_name,
-  jvm_name, js_name, forall_name, family_name :: Name
+  jvm_name, js_name, capi_name, forall_name, family_name :: Name
 export_name     = Ident "export"
 safe_name       = Ident "safe"
 unsafe_name     = Ident "unsafe"
+interruptible_name = Ident "interruptible"
 threadsafe_name = Ident "threadsafe"
 stdcall_name    = Ident "stdcall"
 ccall_name      = Ident "ccall"
@@ -994,6 +817,7 @@ cplusplus_name  = Ident "cplusplus"
 dotnet_name     = Ident "dotnet"
 jvm_name        = Ident "jvm"
 js_name         = Ident "js"
+capi_name       = Ident "capi"
 forall_name     = Ident "forall"
 family_name     = Ident "family"
 
